package org.processmining.dialogs.queueminer.startupremover;

import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.confs.queueminer.startupremover.StartUpRemoveConf;
import org.processmining.models.queueminer.distribution.GraphingData;
import org.processmining.models.queueminer.distribution.Point;

public class RemoverIntervalDialog extends JPanel implements DocumentListener,MouseListener{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	JTextField start;
	JTextField end;
	StartUpRemoveConf surc;
	JTextField suggestion;
	int PAD = 20;
	int topX = PAD;
	int screenWidth;
	int nrPoints = 0;
	
	public RemoverIntervalDialog(final StartUpRemoveConf surc, XLog log){
		this.surc = surc;
		JPanel dayVis = getDayStartsView(log);
		dayVis.addMouseListener(this);
		this.setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
		this.add(dayVis);
		
		JPanel buttons = new JPanel();
		BoxLayout bbl = new BoxLayout(buttons, BoxLayout.LINE_AXIS);
		buttons.setLayout(bbl);
		buttons.setMaximumSize(new Dimension(Integer.MAX_VALUE,20));
		JLabel startLabel = new JLabel("Start day");
		buttons.add(startLabel);
		start = new JTextField();
		start.getDocument().addDocumentListener(this);
		start.getDocument().putProperty("field","updateStart");
		buttons.add(start);
		JLabel endLabel = new JLabel("End day");
		buttons.add(endLabel);
		end = new JTextField();
		end.getDocument().addDocumentListener(this);
		end.getDocument().putProperty("field","updateEnd");
		buttons.add(end);
		
		JLabel suggestionLabel = new JLabel("Day indication (on click):");
		buttons.add(suggestionLabel);
		suggestion  = new JTextField();
		buttons.add(suggestion);
		
		this.add(buttons);
	}
	
	/**
	 * Finds the amount of activity starts per day and plots them
	 * @param log
	 * @return
	 */
	private JPanel getDayStartsView(XLog log){
		XTimeExtension xte = XTimeExtension.instance();
		long subSize = 24 * 3600 * 1000; //a day
		long minTime = Long.MAX_VALUE;
		long maxTime = 0;
		for(XTrace tr:log){
			for(XEvent ev:tr){
				long evTime = xte.extractTimestamp(ev).getTime();
				if(evTime > maxTime){
					maxTime = evTime;
				}
				if(evTime < minTime){
					minTime = evTime;
				}
			}
		}
		int nrBins = (int) ((maxTime-minTime)/subSize)+1;		
		long[] binnedStarts = new long[nrBins];

		for(XTrace tr:log){
			for(XEvent ev:tr){
				long evTime = xte.extractTimestamp(ev).getTime();
				int dayNr = (int) ((evTime-minTime)/subSize);
				binnedStarts[dayNr]++;
			}
		}
		
		ArrayList<Point> points = new ArrayList<>();
		for(int i = 0; i < binnedStarts.length;i++){
			Point p = new Point(i,binnedStarts[i]);
			points.add(p);
		}
		nrPoints = points.size();
		
		List<List<Point>> dataSet = new ArrayList<>();
		dataSet.add(points);
		return new GraphingData(dataSet, PAD, "Day", "Number of activity starts");
	}
	
	private int getDayFromX(int x){
		//System.out.println("screenWidth: "+screenWidth);
		if(x < PAD){
			return 0;
		}
		else if(x > screenWidth-PAD){
			return nrPoints;
		}
		else{//do an actual lookup
			int realX = x-PAD;
			double dayWidth = (double)(screenWidth - 2 * PAD)/(double)nrPoints;
			return (int)(realX/dayWidth);
		}
	}

	public void mouseClicked(MouseEvent arg0) {
		
	}

	public void mouseEntered(MouseEvent arg0) {
		
	}

	public void mouseExited(MouseEvent arg0) {
		
	}

	public void mousePressed(MouseEvent ev) {
		screenWidth = this.getWidth();
		java.awt.Point p = ev.getPoint();
		//System.out.println("pressed at : ("+p.x+","+p.y+")");
		int day = getDayFromX(p.x);
		//System.out.println("day found: "+day);
		suggestion.setText(Integer.toString(day));
	}

	public void mouseReleased(MouseEvent arg0) {
		
	}

	public void changedUpdate(DocumentEvent act) {
		
	}

	public void insertUpdate(DocumentEvent act) {
		switch(act.getDocument().getProperty("field").toString()){
			case "updateStart":
				try{
					int dayStart = Integer.parseInt(start.getText());
					surc.start = dayStart;				}
				catch(Exception e){};
				break;
			case "updateEnd":
				try{
					int dayEnd = Integer.parseInt(end.getText());
					surc.end = dayEnd;				}
				catch(Exception e){};
				break;
		}
	}

	public void removeUpdate(DocumentEvent arg0) {
	}
}
