package org.processmining.models.queueminer.bottleneckfinder;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;

import org.processmining.models.queueminer.distribution.Point;
import org.processmining.models.queueminer.queueclusterer.QueueClustering;
import org.processmining.models.queueminer.utils.Queue;
import org.processmining.models.queueminer.utils.StatisticUtil;

public class BottleneckFinder {
	
	/**
	 * Scores how much a queue duration increases when global business increases
	 * @param q
	 * @return
	 */
	/*private static double scoreQueueGlobal(Queue q, XLog log){
		
	}*/
	
	public static ArrayList<RankedQueue> scoreQueueWeekly(QueueClustering qc){
		//globally rank the parts/contexts by arrival rate
		//first find the arrival count for each part
		HashMap<String,HashMap<Integer,Integer>> arrivalCount = new HashMap<>();
		for(Queue q:qc.getQueues()){
			HashMap<String,HashMap<Integer,Integer>> arrate = q.getArrivalRate();
			for(String context:arrate.keySet()){
				for(Integer k:arrate.get(context).keySet()){
					if(!arrivalCount.containsKey(context)){
						arrivalCount.put(context, new HashMap<Integer,Integer>());
					}
					if(!arrivalCount.get(context).containsKey(k)){
						arrivalCount.get(context).put(k, arrate.get(context).get(k));
					}
					else{
						int oldval = arrivalCount.get(context).get(k);
						arrivalCount.get(context).put(k, oldval+arrate.get(context).get(k));
					}
				}
			}
		}
		//then rank the parts
		ArrayList<RankedPart> arrivalRanking = new ArrayList<>();
		for(String context:arrivalCount.keySet()){
			for(Integer k:arrivalCount.get(context).keySet()){
				arrivalRanking.add(new RankedPart(context,k,arrivalCount.get(context).get(k)));
			}
		}
		Collections.sort(arrivalRanking);
		
		//for each queue, rank parts/context by average service time
		ArrayList<RankedQueue> comparedRanking = new ArrayList<>();
		ArrayList<RankedPart> serviceRanking;
		for(Queue q:qc.getQueues()){
			serviceRanking = new ArrayList<>();
			HashMap<String,HashMap<Integer,ArrayList<Point>>> serviceRates = q.getServiceRate();
			for(String context:serviceRates.keySet()){
				for(Integer k:serviceRates.get(context).keySet()){
					ArrayList<Point> serviceRateDist = serviceRates.get(context).get(k);
					serviceRanking.add(new RankedPart(context,k,getDistMean(serviceRateDist,q.getSubTime())));
				}
			}
			//score = similarity score tussen 2 rankings (kendall tau)
			Collections.sort(serviceRanking);
			//sanitize rankings
			ArrayList<RankedPart> arrivalCopy = new ArrayList<>();
			for(RankedPart r:arrivalRanking){
				for(RankedPart r2:serviceRanking){
					if(r2.getIdentifier().equals(r.getIdentifier())){
						arrivalCopy.add(r);
					}
				}
			}
			
			double score = rankingSimilarity(serviceRanking,arrivalCopy);
			comparedRanking.add(new RankedQueue(q,score));
		}
		Collections.sort(comparedRanking,Collections.reverseOrder());
		return comparedRanking;
	}
	
	/**
	 * Find kendall-tau ranking similarity
	 * @pre rankings are sorted
	 * @param ranking1
	 * @param ranking2
	 * @return
	 */
	private static double rankingSimilarity(ArrayList<RankedPart> ranking1, ArrayList<RankedPart> ranking2){
		assert ranking1.size() == ranking2.size();
		int n = ranking1.size();
		double n0 = 0.5 * n*(n-1);
		//find # concordant pairs & # discordant pairs
		//create map of id's -> rank
		HashMap<Integer,GroupedIndex> groupedIndices = new HashMap<>();
		for(int i = 0; i < ranking1.size(); i++){
			RankedPart rp = ranking1.get(i);
			String curId = rp.getIdentifier();
			int rank2Id = 0;
			for(int j = 0; j < ranking2.size();j++){
				if(ranking2.get(j).getIdentifier().equals(curId)){
					rank2Id = j;
				}
			}
			
			groupedIndices.put(i,new GroupedIndex(i,rank2Id));
		}
		
		int numer = 0;
		for(int i = 1; i< n;i++){
			for(int j = 0; j < i-1; j++){
				GroupedIndex ii = groupedIndices.get(i);
				GroupedIndex ji = groupedIndices.get(j);
				numer += sign(ii.x - ji.x) * sign(ii.y - ji.y); 
			}
		}
		
		 
		return numer/n0;
	}
	
	private static int sign(int nr){
		if(nr > 0){
			return 1;
		}
		else if(nr < 0){
			return -1;
		}
		else{
			return 0;
		}
	}
	
	/**
	 * Find the mean value given the dist represented by a list of points
	 * @param dist
	 * @return
	 */
	public static double getDistMean(ArrayList<Point> dist, long partSize){
		ArrayList<Long> longList = new ArrayList<>();
		for(int i = 0; i < dist.size(); i++){
			long x = dist.get(i).getX();
			long y = dist.get(i).getY();
			for(int j = 0; j < y; j++){
				longList.add(x * partSize);
			}
		}
		return StatisticUtil.getMean(longList);
	}
}
