package org.processmining.models.queueminer.context;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class ContextInfoParser {
	
	public static TimeContext readTimeContext(InputStream input) throws Exception{
		HashMap<String,ArrayList<ContextItem>> cismap = new HashMap<>();
		//try{
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document doc = dBuilder.parse(input);
			doc.getDocumentElement().normalize();
			//find all context items 
			NodeList vars = doc.getElementsByTagName("contextVariable");
			//System.out.println("vars length:"+vars.getLength());
			for(int j = 0; j  < vars.getLength(); j++){
				ArrayList<ContextItem> cis = new ArrayList<>();
				Node var = vars.item(j);
				String varName = var.getAttributes().getNamedItem("name").getTextContent();
				Element vElement = (Element) var;
				NodeList nList = vElement.getElementsByTagName("contextItem");
				//System.out.println("nList length: "+nList.getLength());
				//System.out.println("nList: "+nList.toString());
				for(int i = 0; i < nList.getLength(); i++){
					Node nNode = nList.item(i);
					if (nNode.getNodeType() == Node.ELEMENT_NODE) {
						Element eElement = (Element) nNode;
						String date = eElement.getElementsByTagName("time").item(0).getAttributes().item(0).getNodeValue();
						String id = eElement.getElementsByTagName("context").item(0).getAttributes().item(0).getNodeValue();
						DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.ENGLISH);
					    long time =  df.parse(date).getTime(); 
						ArrayList<String> idl = new ArrayList<>();
						idl.add(id);
						ContextItem ci = new ContextItem(time,idl);
						cis.add(ci);
					}
				}
				//System.out.println("put stuff in cismap");
				cismap.put(varName, cis);
			}
			
		/*}
		catch(Exception e){
			System.err.println("Context cannot be read: "+e);
		}*/
		return new TimeContext(cismap);
	}
	
	public static void writeTimeContext(TimeContext tc, File file){
		Document dom;
	    Element e = null;

	    // instance of a DocumentBuilderFactory
	    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	    try {
	        // use factory to get an instance of document builder
	        DocumentBuilder db = dbf.newDocumentBuilder();
	        // create instance of DOM
	        dom = db.newDocument();
	        //for every context info variable
	        Element contextInfo = dom.createElement("contextInfo");
	        for(String key: tc.items.keySet()){
	        	Element contextVar = dom.createElement("contextVariable");
	        	contextVar.setAttribute("name", key);
	        	ArrayList<ContextItem> items = tc.items.get(key);
	        	//for all items within that variable
	        	for(ContextItem ci:items){
	        		Element citem = dom.createElement("contextItem");
	        		//add context identifiers
	        		for(String ident: ci.identifiers){
	        			e = dom.createElement("context");
	        			e.setAttribute("value", ident);
	        			citem.appendChild(e);
	        		}
	        		//add time
	        		DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.ENGLISH);
	        		Date date = new Date(ci.time);
	        		String formatDate =  df.format(date);
	        		e = dom.createElement("time");
	        		e.setAttribute("value",formatDate);
	        		citem.appendChild(e);
	        		contextVar.appendChild(citem);
	        	}
	        	contextInfo.appendChild(contextVar);        	
	        }
	        dom.appendChild(contextInfo);	
        	//write
	        try {
	            Transformer tr = TransformerFactory.newInstance().newTransformer();
	            tr.setOutputProperty(OutputKeys.INDENT, "yes");
	            tr.setOutputProperty(OutputKeys.METHOD, "xml");
	            tr.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
	            //tr.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM, "roles.dtd");
	            tr.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");
	            // send DOM to file
	            tr.transform(new DOMSource(dom), 
	                                 new StreamResult(new FileOutputStream(file)));

	        } catch (TransformerException te) {
	            System.out.println(te.getMessage());
	        } catch (IOException ioe) {
	            System.out.println(ioe.getMessage());
	        }
		} catch (ParserConfigurationException pce) {
	        System.out.println("UsersXML: Error trying to instantiate DocumentBuilder " + pce);
	    }
	}
}
