package org.processmining.models.queueminer.context;

import java.util.ArrayList;
import java.util.HashMap;

public class TimeContext {
	public HashMap<String,ArrayList<ContextItem>> items;

	public TimeContext(HashMap<String,ArrayList<ContextItem>> items){
		this.items = items;
	}
	
	public HashMap<String,ArrayList<ContextItem>> getItems() {
		return items;
	}
	
	/**
	 * Appends one tc file to another
	 * @param tc1
	 * @param tc2
	 * @return
	 */
	public static TimeContext simpleMerge(TimeContext tc1, TimeContext tc2){
		HashMap<String,ArrayList<ContextItem>> resultingItems = new HashMap<>();
		//System.out.println("tc1");
		for(String key:tc1.items.keySet()){
			//System.out.println("putting for key: "+key);
			resultingItems.put(key, tc1.items.get(key));
		}
		//System.out.println("tc2");
		for(String key:tc2.items.keySet()){
			//System.out.println("putting for key: "+key);
			resultingItems.put(key, tc2.items.get(key));
		}
		return new TimeContext(resultingItems);
	}
	
	/**
	 * Adds the unique times and identifiers and merges them with the current map
	 * New times are introduced as necessary
	 * @param tc
	 */
	public ConcatenatedContext concatenateAll(){
		ArrayList<ContextItem> newList = new ArrayList<>();
		int nrVars = items.size();
		ContextItem[] lastItems = new ContextItem[nrVars];
		int[] itemCounters = new int[nrVars];
		boolean[] itemsDone = new boolean[nrVars];
		for (int i = 0; i < itemsDone.length; i++) {
			itemsDone[i] = false;
		}
		for (int j = 0; j < itemCounters.length; j++) {
			itemCounters[j] = 0;
		}
		int[] itemSizes = new int[nrVars];
		//Make a list with an item for each combination
		int c = 0;
		for(String key:items.keySet()){
			lastItems[c] = items.get(key).get(itemCounters[c]);
			itemSizes[c] = items.get(key).size();
			c++;
		}
		long time = Long.MAX_VALUE;
		ArrayList<String> ids = new ArrayList<>();
		for(ContextItem lasti:lastItems){
			time = Math.min(time, lasti.time);
			ids.addAll(lasti.identifiers);
		}
		ContextItem newItem = new ContextItem(time,ids);
		newList.add(newItem);
		
		boolean done = false;
		while(!done){
			//add entry to concatenated list
			time = 0;
			long smallTime = Long.MAX_VALUE;
			ids = new ArrayList<>();
			int smallestItem = 0;
			for(int i = 0; i < lastItems.length;i++){
				long itime = lastItems[i].time;
				time = Math.max(time, itime);
				if(itime < smallTime && !itemsDone[i]){
					smallTime = itime;
					smallestItem = i;
					if(itemCounters[i] == itemSizes[i]-1){
						itemsDone[i] = true;
					}
				}
				//add all items
				ids.addAll(lastItems[i].identifiers);
			}
			newItem = new ContextItem(time,ids);
			newList.add(newItem);
			//increase the counter for the item with the smallest time
			//which is not yet at the end
			if(itemCounters[smallestItem] < itemSizes[smallestItem]-1){
				itemCounters[smallestItem]++;
			}
			//update last items
			c = 0;
			for(String key:items.keySet()){
				lastItems[c] = items.get(key).get(itemCounters[c]);
				c++;
			}

			done = true;
			//done if all itemCounters are at the last item
			for(int i = 0; i < lastItems.length; i++){
				done = done && itemsDone[i];
			}
		}
		return new ConcatenatedContext(newList);
	}
}
