package org.processmining.models.queueminer.distribution;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.font.FontRenderContext;
import java.awt.font.LineMetrics;
import java.awt.geom.Line2D;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JPanel;
 
public class GraphingData extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	List<List<Point>> data;
	int PAD = 20;
	String xLabel;
	String yLabel;
	
	public GraphingData(List<List<Point>> data2, int PAD, String xLabel, String yLabel){
		this.data = data2;
		this.PAD = PAD;
		this.xLabel = xLabel;
		this.yLabel = yLabel;
	}
 
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D)g;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                            RenderingHints.VALUE_ANTIALIAS_ON);
        int w = getWidth();
        int h = getHeight();
        // Draw ordinate.
        g2.draw(new Line2D.Double(PAD, PAD, PAD, h-PAD));
        // Draw abcissa.
        g2.draw(new Line2D.Double(PAD, h-PAD, w-PAD, h-PAD));
        // Draw labels.
        Font font = g2.getFont();
        FontRenderContext frc = g2.getFontRenderContext();
        LineMetrics lm = font.getLineMetrics("0", frc);
        float sh = lm.getAscent() + lm.getDescent();
        // Ordinate label.
        String s = yLabel;
        float sy = PAD + ((h - 2*PAD) - s.length()*sh)/2 + lm.getAscent();
        for(int i = 0; i < s.length(); i++) {
            String letter = String.valueOf(s.charAt(i));
            float sw = (float)font.getStringBounds(letter, frc).getWidth();
            float sx = (PAD - sw)/2;
            g2.drawString(letter, sx, sy);
            sy += sh;
        }
        // Abcissa label.
        s = xLabel;
        sy = h - PAD + (PAD - sh)/2 + lm.getAscent();
        float sw = (float)font.getStringBounds(s, frc).getWidth();
        float sx = (w - sw)/2;
        g2.drawString(s, sx, sy);
        //System.out.println("drawing lines");
        // Draw lines.
        long minX = getMinX();
        long maxX = getMaxX();
        
        double xScale = (double)(w - 2*PAD)/(maxX-minX);
        double yScale = (double)(h - 2*PAD)/getMaxY();
        
        List<Color> colorList = getColorList(data.size());
        
        for(int i = 0; i < data.size(); i++) {
        	g2.setPaint(colorList.get(i));
        	for(int j = 0; j < data.get(i).size()-1; j++){
	            double x1 = PAD + (xScale*data.get(i).get(j).x) - (minX*xScale);
	            double y1 = h - PAD - (yScale*data.get(i).get(j).y);
	            double x2 = PAD + (xScale*data.get(i).get(j+1).x) - (minX*xScale);
	            double y2 = h - PAD - (yScale*data.get(i).get(j+1).y);
	            g2.draw(new Line2D.Double(x1, y1, x2, y2));
        	}
        }
        //System.out.println("drawing points");
        // Mark data points.
        /*g2.setPaint(Color.red);
        for(int i = 0; i < data.size(); i++) {
            double x = PAD + (xScale*data.get(i).x);
            double y = h - PAD - yScale*data.get(i).y;
            g2.fill(new Ellipse2D.Double(x-2, y-2, 4, 4));
        }
        System.out.println("drawing labels");
        //Add axis labels
        /*g2.setPaint(Color.black);
        long minX = getMinX();
        long maxX = getMaxX();
        Font curFont = g2.getFont();
        g2.setFont(new Font(curFont.getFontName(),curFont.getStyle(),8));
        for(int i = 0; i < data.size();i++){
        	if(i%5 == 0){
        		g2.drawString(data.get(i).x+"", (int) (PAD + (xScale*data.get(i).x)), h);
        	}
        }
        /*for(int i = 0; i < maxX-minX; i++){
        	 g2.drawString(""+i, PAD+i*((w-2*PAD)/(maxX-minX)), h-PAD);
        }*/
        //Write min and max value
        g2.setPaint(Color.black);
        Font curFont = g2.getFont();
        g2.setFont(new Font(curFont.getFontName(),curFont.getStyle(),16));
        g2.drawString(data.get(0).get(0).x+"", PAD, h);
        long maxVall = 0;
        for(Point p:data.get(0)){
        	if(p.x > maxVall){
        		maxVall = p.x;
        	}
        }
        
        String maxVal = maxVall+"";
        g2.drawString(maxVal, w-PAD-(maxVal.length() * curFont.getSize()), h);
        g2.drawString(getMaxY()+"", PAD, PAD);
    }
    
    public List<Color> getColorList(int size){
    	List<Color> colorList = new ArrayList<>();
        colorList.add(Color.blue);
        colorList.add(Color.red);
        colorList.add(Color.green);
        colorList.add(Color.cyan);
        colorList.add(Color.magenta);
        if(size > 5){
        	for(int i = 0; i < size - 5; i++){
        		float part = i/(float)(size-5);
        		colorList.add(Color.getHSBColor(part,part,part));
        	}
        }
        return colorList;
    }
    
    private long getMinX() {
    	long min = Long.MAX_VALUE;
    	for(int i = 0; i < data.size(); i++) {
        	for(int j = 0; j < data.get(i).size(); j++){
        		if(data.get(i).get(j).x < min){
        			min = data.get(i).get(j).x;
            	}
        	}
        }
        return min;
    }
 
    private long getMaxX() {
    	long max = Long.MIN_VALUE;
        for(int i = 0; i < data.size(); i++) {
        	for(int j = 0; j < data.get(i).size(); j++){
        		if(data.get(i).get(j).x > max){
        			max = data.get(i).get(j).x;
            	}
        	}
        }
        return max;
    }
    
    private long getMaxY() {
    	long max = -Long.MAX_VALUE;
    	for(int i = 0; i < data.size(); i++) {
        	for(int j = 0; j < data.get(i).size(); j++){
        		if(data.get(i).get(j).y > max){
        			max = data.get(i).get(j).y;
            	}
        	}
        }
        return max;
    }
}
