package org.processmining.models.queueminer.logcontext;

import java.util.ArrayList;
import java.util.HashMap;

import org.deckfour.xes.model.XLog;
import org.processmining.models.queueminer.context.ContextItem;
import org.processmining.models.queueminer.context.TimeContext;
import org.processmining.models.queueminer.utils.StatisticUtil;
import org.processmining.models.queueminer.utils.TimeActivity;
import org.processmining.models.queueminer.utils.TimeActivityUtilities;
import org.processmining.models.queueminer.utils.TimeUtilities;

public class LogWorkloadContextFinder {
	
	public static TimeContext findWorkLoadInformation(XLog log){
		ArrayList<TimeActivity> tlog = TimeActivityUtilities.findTimeActivities(log);
		//for every day that is relevant for the log: measure the amount of events in a window
		// of 3days in the future and in the past
		//find the mean, std, max, min of overlaps
		//set the workload at a given hour to:
		//	- Busy (> mean+std)
		//	- Quiet (< mean-std)
		//	- Normal (otherwise)
		long firstDay  = TimeUtilities.findFirstDayStart(tlog);
		long lastDay = TimeUtilities.findLastDayStart(tlog);
		ArrayList<TimeActivity> window = new ArrayList<>();
		//Find initial window (first 4 days)
		long dayLength = 24 * 3600 * 1000;
		int nrDays = (int)((lastDay-firstDay)/(dayLength));
		int[] workload = new int[nrDays];
		int dayRadius = 3;
		long wStart = firstDay;
		long wEnd = wStart + (dayRadius+1) * dayLength;
		//Find initial window events
		boolean done = false;
		int c = 0;
		while(!done){
			TimeActivity ta = tlog.get(c);
			long taStart = ta.start;
			if(taStart >= wStart && taStart <= wEnd){
				window.add(ta);
				c++;
			}
			else{
				done = true;
			}
		}
		workload[0] = window.size();
		//For every following day, update the window and save the rsult
		for(int i = 1; i < nrDays;i++){
			//update window range
			if(i > dayRadius){
				wStart = firstDay + (i-dayRadius) * dayLength;
			}
			else{
				wStart = firstDay;
			}
			
			if(i < nrDays-dayRadius){
				wEnd = firstDay + ((i+dayRadius+1) * dayLength);
			}
			else{
				wEnd = lastDay+dayLength;
			}
			//remove stuff from window that is outside range
			ArrayList<TimeActivity> newWindow = new ArrayList<>();
			for(TimeActivity wta:window){
				if(wta.start >= wStart && wta.start <= wEnd){
					newWindow.add(wta);
				}
			}
			window = newWindow;
			//add new stuff to window that has come into range
			done =false;
			while(!done && c < tlog.size()-2){
				TimeActivity ta = tlog.get(c);
				long taStart = ta.start;
				if(taStart >= wStart && taStart <= wEnd){
					window.add(ta);
					c++;
				}
				else{
					done = true;
				}
			}
			//add new workload entry
			workload[i] = window.size();
		}
		
		//Find the mean and std of the workload array
		double mean = StatisticUtil.getMean(workload);
		double std = StatisticUtil.getStdDev(workload);
		ArrayList<ContextItem> cis = new ArrayList<>();
		for(int i = 0; i < workload.length; i++){
			long time = firstDay + i*dayLength;
			int wl = workload[i];
			String context = "Regular";
			if(wl < mean-std){
				context = "Quiet";
			}
			else if (wl > mean+std){
				context = "Busy";
			}
			ArrayList<String> cs = new ArrayList<>();
			cs.add(context);
			cis.add(new ContextItem(time,cs));
		}
		
		HashMap<String,ArrayList<ContextItem>> cismap = new HashMap<>();
		cismap.put("Workload", cis);
		return new TimeContext(cismap);
	}
	 
	
	/*public static XLog addContextInformation(XLog log){
		ArrayList<TimeActivity> tlog = TimeActivityUtilities.findTimeActivities(log);
		int[] overlaps = new int[tlog.size()];
		int maxOverlap = 0;
		for(int i = 0; i < tlog.size(); i++){
			int counter = 0;
			for(TimeActivity ta2:tlog){
				long start = tlog.get(i).start;
				if(!tlog.get(i).equals(ta2)){
					long otherS = ta2.start;
					long otherE = ta2.end;
					if(start >= otherS && start <= otherE){
						counter++;
					}
				}
			}
			overlaps[i] = counter;
			if(counter > maxOverlap){
				maxOverlap = counter;
			}
			System.out.println("trace "+i+ "of "+tlog.size()+" done");
		}
		
		//overlaps[] contains amount of overlaps for events, maxOverlap is maximum
		int busyLimit = (int) (maxOverlap * 0.75);
		
		XExtensionManager xem = XExtensionManager.instance();
		URI conceptUri = URI.create("http://www.zerodivision.nl/XES/context.xesext");
		XExtension xce = xem.getByUri(conceptUri);
		Collection<XAttribute> attributes = xce.getEventAttributes();
		
		XFactory xlogf = XFactoryRegistry.instance().currentDefault();
        XLog xlog = xlogf.createLog();
        XConceptExtension xcone = XConceptExtension.instance();
        int counter = 0;
        for(XTrace tr:log){
			XTrace xtr = xlogf.createTrace();
			xcone.assignName(xtr, tr.getAttributes().get("concept:name").toString());
			for(XEvent ev:tr){
				//get context key
				XAttribute contattr = attributes.iterator().next();
				String key = contattr.getKey();
				//get context value
				int index = indexOfXEvent(ev,tlog);
				String context = overlaps[index] >= busyLimit ? "Busy" : "Quiet";
				XAttribute value = new XAttributeLiteralImpl(key, context);
				//add new info to attribute map of event
				XAttributeMap evattr = ev.getAttributes();
				evattr.put(key, value);
				ev.setAttributes(evattr);
				
				xtr.add(ev);
			}
			xlog.add(tr);
			System.out.println("trace "+counter+" "+log.size());
			counter++;
        }
		return xlog;	
	}
	
	public static int indexOfXEvent(XEvent ev, ArrayList<TimeActivity> tlog){
		int index = 0;
		for(int i = 0; i < tlog.size(); i++){
			if(tlog.get(i).original.equals(ev)){
				index = i;
			}
		}
		return index;
	}*/
}
