package org.processmining.models.queueminer.matrixfinder;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XOrganizationalExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.framework.plugin.Progress;

public class MatrixFinder {
	
	public static ResActMatrix findMatrix(XLog log, Progress p){
		HashMap<String, Integer> actMap = new HashMap<>();
		HashMap<String, Integer> resMap = new HashMap<>();
		//First find all the activities and resources
		XConceptExtension ce = XConceptExtension.instance();
		XOrganizationalExtension oe = XOrganizationalExtension.instance();
		int actCounter = 0;
		int resCounter = 0;
		int progressc = 0;
		for(XTrace t:log){
			p.setValue(progressc);
			for(int i = 0; i < t.size(); i++){
				XEvent ev = t.get(i);
				String act = ce.extractName(ev);
				if(!actMap.containsKey(act)){
					actMap.put(act, actCounter);
					actCounter++;
				}
				String res = oe.extractResource(ev);
				if(!resMap.containsKey(res)){
					resMap.put(res, resCounter);
					resCounter++;
				}
			}
			progressc++;
		}
		//Make act and res stringarrays
		String[] actNames = new String[actMap.size()];
		String[] resNames = new String[resMap.size()];
		Iterator<Entry<String, Integer>> it = actMap.entrySet().iterator();
	    while (it.hasNext()) {
	    	Entry<String, Integer> pairs = it.next();
	    	int pos = pairs.getValue();
	    	actNames[pos] = pairs.getKey();
	    }
	    Iterator<Entry<String, Integer>> it2 = resMap.entrySet().iterator();
	    while (it2.hasNext()) {
	    	Entry<String, Integer> pairs = it2.next();
	    	int pos = pairs.getValue();
	    	resNames[pos] = pairs.getKey();
	    }
		
		//Then count the amount of times activities are performed by certain resources
		int[][] matrix = new int[actMap.size()][resMap.size()];
		for(XTrace t:log){
			for(int i = 0; i < t.size(); i++){
				XEvent ev = t.get(i);
				String act = ce.extractName(ev);
				String res = oe.extractResource(ev);
				matrix[actMap.get(act)][resMap.get(res)] ++;
			}
		}
		
		/* //USEFUL TEST PRINTS
		System.out.println("resnames: ");
		for(String rname:resNames){
			System.out.println(rname);
		}
		
		System.out.println("matrix:");
		for(int i =0 ; i < matrix.length; i++){
			for(int j = 0; j < matrix[i].length; j++){
				System.out.print(matrix[i][j]+"|");
			}
			System.out.println();
		}*/
		
		return new ResActMatrix(resNames,actNames,matrix);
	}
}
