package org.processmining.models.queueminer.patternvisualizer;

import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.processmining.models.queueminer.utils.StatisticUtil;

public class PLTVNode{
	PrimeLogTreeView parent;
	PrimeLogNode node;
	
	public PLTVNode(PrimeLogTreeView parent, PrimeLogNode node){
		this.parent = parent;
		this.node = node;
	}
	
	public void draw(Graphics g, int x1, int y1, int width, int height, double maxValue, String metric){
		ArrayList<DistributionInfo> dists = node.getSubLogs();
		int distWidth = (int)(width/(double)dists.size());
		int drawnCounter = 0;
		for(int i  =0; i < dists.size(); i++){
				switch(metric){
				case "Distribution size":
					int dSize = GranularityFindingUtils.findPointListSize(dists.get(i).distribution);
					//System.out.println("dSize: "+dSize+", maxVal: "+maxValue);
					g.setColor(findMatchingColor(dSize,maxValue));
					break;
				case "Distribution mean":
					DistributionInfo info = dists.get(i);
					double[] data = StatisticUtil.pointsToDoubleArray(info.distribution);
					double dMean = StatisticUtil.getMean(data);
					int weightedMean = (int)(dMean * node.getK());
					g.setColor(findMatchingColor(weightedMean,maxValue));
					break;
				case "Distribution variance":
					DistributionInfo info2 = dists.get(i);
					double[] data2 = StatisticUtil.pointsToDoubleArray(info2.distribution);
					double dVar = StatisticUtil.getVariance(data2);
					g.setColor(findMatchingColor(dVar,maxValue));
					break;
				default:
					int defSize = GranularityFindingUtils.findPointListSize(dists.get(i).distribution);
					g.setColor(findMatchingColor(defSize,maxValue));
					break;
				}
				g.fillRect(x1+drawnCounter*distWidth, y1, distWidth, height);
				//Indication of threshold reached
				if(dists.get(i).approved){
					g.setColor(Color.magenta);
				}
				else{
					g.setColor(Color.green);
				}
				g.fillRect(x1+drawnCounter*distWidth, y1+(int)(height*0.8), distWidth, (int)(height-height*0.8));
				drawnCounter++;
		}
		int sizeUsed = dists.size() * distWidth;
		int sizeLeft = height - sizeUsed+1;
		g.setColor(Color.darkGray);
		g.fillRect(x1+sizeUsed, y1, sizeLeft, height);
		//g.setColor(Color.black);
		//g.drawRect(x1-1, y1-1, size+1, size+1);
		g.setColor(Color.blue);
		g.drawString(Integer.toString(node.getK()), x1+3, y1+3+g.getFont().getSize());
	}
	
	private Color findMatchingColor(double value, double max){
		double value10Log = Math.log10(value+1); //avoid negative number in log 
		double max10Log = Math.log10(max+1);
		double frac = value10Log/max10Log;
		int totalCol = (int)(frac * 3 * 255);
		/*if(node.getK() == 24 || node.getK() == 12){
			System.out.println("value: "+value);
			//System.out.println("totalCol: "+totalCol);
			System.out.println("max: "+max);
		}*/
		int r = totalCol > 255? 255: totalCol;
		int g = 0;
		if(totalCol > 255){
			if(totalCol > 255*2){
				g = 255;
			}
			else{
				g = totalCol-255;
			}
		}
		int b = totalCol > 255*2? totalCol-255*2: 0;
		Color c = new Color(r,g,b);
		return c;
		//return Color.getHSBColor(120f, frac, frac);
	}
	
	public JComponent getPLTVNode(){
		JPanel nodeLayout = new JPanel();
		JLabel kLabel = new JLabel(Integer.toString(node.getK()));
		nodeLayout.add(kLabel);
		//show all log sizes:
		/*JLabel dbCount;
		for(DistributionInfo db:node.getSubLogs()){
			dbCount = new JLabel(Integer.toString(GranularityFindingUtils.findPointListSize(db.distribution)));
			nodeLayout.add(dbCount);
		}*/
		
		nodeLayout.setBackground(Color.green);

		return nodeLayout;
	}
	
	
}
