package org.processmining.models.queueminer.patternvisualizer;

import java.util.ArrayList;

import org.processmining.models.queueminer.distribution.DistributionFilter;
import org.processmining.models.queueminer.distribution.Point;
import org.processmining.models.queueminer.utils.TimeActivity;

public class PrimeLogTree {
	private int daysConsidered = 7;
	private long intervalTime = daysConsidered * 24 * 3600 * 1000;
	private PrimeLogNode root;
	private ArrayList<PrimeLogNode> nodes;
	private int threshold;
	private int[] primesUsed;
	private long weekStart;
	private ArrayList<TimeActivity> logUsed;
	
	public PrimeLogTree(int threshold, ArrayList<TimeActivity> tal, int[] primesUsed, long weekStart){
		this.primesUsed = primesUsed;
		this.threshold = threshold;
		this.weekStart = weekStart;
		this.logUsed = tal;
		init();
	}
	
	private void init(){
		nodes = new ArrayList<>();
		
		//Find the information of the root log distribution
		long start = DistributionFilter.logMinDuration(logUsed);
		long end = DistributionFilter.logMaxDuration(logUsed);
		long binSize = intervalTime/(daysConsidered*1);
		ArrayList<Point> rootdist = DistributionFilter.findDistribution(logUsed, binSize);
		int distSize = GranularityFindingUtils.findPointListSize(rootdist);
		boolean approved = (distSize >= threshold);
		DistributionInfo rootdi = new DistributionInfo(rootdist, logUsed, approved, start, end);
		
		//Add the root node
		ArrayList<DistributionInfo> rootdil = DistributionFilter.splitRootDist(rootdi,daysConsidered,threshold,intervalTime,weekStart);
		PrimeLogNode root = new PrimeLogNode(1,rootdil);
		this.root = root;
		this.nodes.add(root);
	}
	
	public void newTree(int daysConsidered){
		this.daysConsidered = daysConsidered;
		init();
	}
	
	public void expandNode(int k){
		if(!nodeExists(k)){
			System.err.println("Somehow a non-existing node with k = "+k+" was to be expanded.");
			return;
		}
		//Find the node
		PrimeLogNode parent = nodes.get(0);//There should be some parent found, since k exists
		for(PrimeLogNode pln:nodes){
			if(pln.k == k){
				parent = pln;
			}
		}
		//Return if parent has no more approved parts
		
		boolean noneApproved = true;
		for(DistributionInfo db:parent.sublogs){
			noneApproved = (noneApproved && !db.approved);
		}
		if(noneApproved){
			//maybe give the user some message here
			//System.out.println("No children are approved");
			return;
		}
		
		//a new child should be introduced for each prime product
		//although only for the approved subparts
		for(int p:primesUsed){
			int nextK = p * k;
			if(!nodeExists(nextK)){
				//System.out.println("k = "+k+" ,nextK = "+nextK);
				long binSize = intervalTime/(daysConsidered*nextK);
				//split each of the approved subparts and add them to a list
				ArrayList<DistributionInfo> childrenInfo = new ArrayList<>();
				int counter = 0;
				for(DistributionInfo di:parent.sublogs){
					if(di.approved){
						counter++;
						ArrayList<DistributionInfo> split = DistributionFilter.splitDistributionInfo(di,p,threshold,intervalTime,binSize,weekStart);
						childrenInfo.addAll(split);
					}
					else{
						//add empty dummy columns
						ArrayList<Point> dummyDist = new ArrayList<>();
						ArrayList<TimeActivity> dummytal = new ArrayList<>();
						DistributionInfo dummy = new DistributionInfo(dummyDist, dummytal, false, 0, 0);
						for(int i = 0; i < p;i++){
							childrenInfo.add(dummy);
						}
					}
				}
				//System.out.println(counter+" were good");
				//Create new child and add it
				PrimeLogNode child = new PrimeLogNode(nextK,childrenInfo);
				parent.addChild(child);
				nodes.add(child);
			}
		}
	}
	
	public int getDaysConsidered(){
		return daysConsidered;
	}
	
	public ArrayList<PrimeLogNode> getNodes(){
		return nodes;
	}
	
	public PrimeLogNode getRoot(){
		return root;
	}
	
	private boolean nodeExists(int k){
		for(PrimeLogNode pln:nodes){
			if(pln.k == k){
				return true;
			}
		}
		return false;
	}
}
