package org.processmining.models.queueminer.patternvisualizer;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.JPanel;

import org.processmining.models.queueminer.distribution.Point;
import org.processmining.models.queueminer.utils.MathUtil;
import org.processmining.models.queueminer.utils.StatisticUtil;

public class PrimeLogTreeView extends JPanel implements MouseListener{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1772868446349275393L;
	private int nodeWidth = 150;
	private int nodeHeight = 100;
	private int layerSpacing = 20;
	private String metric = "Distribution size";
	private static double maxValue;
	private String zoomOption = "none";
	private ArrayList<Integer> zoomNodes = new ArrayList<>();
	private int cRootNr = 1;
	private boolean drawdummies = true;
	
	private PrimeLogTree basis;
	HashMap<Integer,Point> nodeMap;
	
	public void updateParameters(int layerSpacing, int nodeWidth, int nodeHeight, String metric, boolean dummies){
		this.layerSpacing = layerSpacing;
		this.nodeWidth = nodeWidth;
		this.nodeHeight = nodeHeight;
		this.metric = metric;
		drawdummies = dummies;
	}
	
	public PrimeLogTreeView(PrimeLogTree basis){
		this.basis = basis;
		this.addMouseListener(this);
		nodeMap = new HashMap<>();
		maxValue = GranularityFindingUtils.findPointListSize(basis.getRoot().getSubLogs().get(0).distribution);
		
		//this.setLayout(new BoxLayout(this,BoxLayout.PAGE_AXIS));
	}
	
	public void newTree(int daysConsidered){
		basis.newTree(daysConsidered);
		nodeMap = new HashMap<>();
		maxValue = GranularityFindingUtils.findPointListSize(basis.getRoot().getSubLogs().get(0).distribution);
		this.repaint();
	}
	/*public void drawTree(){	
		super.removeAll();
		PrimeLogNode root = basis.getRoot();
		PLTVNode rootn = new PLTVNode(this,root);
		JPanel layer = new JPanel();
		layer.add(rootn.getPLTVNode());
		
		JPanel layer2 = new JPanel();
		drawSubNodes(root,layer2);
		
		this.add(layer);
		this.add(layer2);
		super.revalidate();
	}
	
	private void drawSubNodes(PrimeLogNode pln, JPanel panel){
		for(PrimeLogNode child:pln.getChildren()){
			PLTVNode newchild = new PLTVNode(this,child);
			drawSubNodes(child,panel);
			super.add(newchild.getPLTVNode());
		}
	}*/
	
	public boolean shouldDrawDummies(){
		return drawdummies;
	}
	
	public PrimeLogTree getBasis(){
		return basis;
	}
	
	@Override public void paintComponent(Graphics g){
		super.paintComponent(g);
		Dimension dimension = this.getSize();
		g.setColor(new Color(30,30,100));
		g.fillRect(0, 0, dimension.width, dimension.height);
		
		updateMax();
		if(zoomOption.equals("Node set")){
			ArrayList<PrimeLogNode> realNodes = new ArrayList<>();
			for(PrimeLogNode pln: basis.getNodes()){
				if(zoomNodes.contains(pln.getK())){
					realNodes.add(pln);
				}
			}
			//find the least common multiple of all dist sizes
			ArrayList<Integer> numbers = new ArrayList<>();
			for(Integer z:zoomNodes){
				int distSize = z * basis.getDaysConsidered();
				numbers.add(distSize);
			}
			int LCM = MathUtil.leastCommonMultiple(numbers);
			while(LCM > dimension.width){
				LCM *= 0.5;
				System.err.println("Too large LCM between the nodes");
			}
			
			nodeWidth = LCM;
			layerSpacing = 20;
			nodeHeight = (dimension.height-((zoomNodes.size()+1)*layerSpacing))/zoomNodes.size();
			
			for(int i = 0; i < realNodes.size(); i++){
				PrimeLogNode child = realNodes.get(i);
				int thisX = dimension.width/2 - nodeWidth/2;
				int thisY = i * (layerSpacing + nodeHeight);
				PLTVNode rootview = new PLTVNode(this,child);
				rootview.draw(g, thisX,thisY,nodeWidth,nodeHeight,maxValue,metric);
			}
		}
		else{
			PrimeLogNode root = basis.getRoot();
			if(zoomOption.equals("Single node")){
				for(PrimeLogNode n:basis.getNodes()){
					if(n.getK() == cRootNr){
						root = n;
					}
				}
			}
			
			PLTVNode rootview = new PLTVNode(this,root);
			
			rootview.draw(g, dimension.width/2-nodeWidth/2, 5,nodeWidth,nodeHeight,maxValue,metric);
			Point rootp = new Point(dimension.width/2-nodeWidth/2,5);
			nodeMap = new HashMap<>();
			nodeMap.put(root.getK(),rootp);
			int nrChildren = root.getChildren().size();
			if(nrChildren > 0){
				paintChildren(g, root,dimension.width);
			}
		}
	}
	
	public void paintChildren(Graphics g, PrimeLogNode parent, int widthAvailable){
		int prevY = (int) nodeMap.get(parent.getK()).getY();
		int prevX = (int) nodeMap.get(parent.getK()).getX();
		int thisY = prevY+layerSpacing+nodeHeight;
		int nrChildren = parent.getChildren().size();
		int leftBound = prevX - widthAvailable/2;
		int subWidth = widthAvailable/nrChildren;
		for(int i = 0; i < nrChildren; i++){
			PrimeLogNode child = parent.getChildren().get(i);
			int thisK = child.getK();
			int thisX = leftBound + subWidth * i + subWidth/2;
			PLTVNode rootview = new PLTVNode(this,child);
			rootview.draw(g, thisX,thisY,nodeWidth,nodeHeight,maxValue,metric);
			Point childLoc = new Point(thisX,thisY);
			nodeMap.put(thisK, childLoc);
			//Draw children
			int cnrChildren = child.getChildren().size();
			if(cnrChildren > 0){
				paintChildren(g, child,subWidth);
			}
			
		}
	}
	
	public void updateMax(){
		//Update the maximum value
		switch(metric){
			case "Distribution size":
				int curMax = 0;
				ArrayList<DistributionInfo> sublogs = basis.getRoot().getSubLogs();
				for(int i = 0; i < sublogs.size();i ++){
					curMax += GranularityFindingUtils.findPointListSize(sublogs.get(i).distribution);
				}
				maxValue = curMax;
				break;
			case "Distribution mean":
				int maxMean = 0;
				for(PrimeLogNode pltvn: basis.getNodes()){
					for(DistributionInfo di:pltvn.getSubLogs()){
						double[] data = StatisticUtil.pointsToDoubleArray(di.distribution);
						double distMean = StatisticUtil.getMean(data);
						if(distMean*pltvn.getK() > maxMean){
							maxMean = (int)(distMean*pltvn.getK());
						}
					}
				}
				maxValue = maxMean;
				break;
			case "Distribution variance":
				int maxVar = 0;
				for(PrimeLogNode pltvn: basis.getNodes()){
					for(DistributionInfo di:pltvn.getSubLogs()){
						double[] data = StatisticUtil.pointsToDoubleArray(di.distribution);
						int distVar = (int)StatisticUtil.getVariance(data);
						if(distVar > maxVar){
							maxVar = distVar;
						}
					}
				}
				//System.out.println("maxVar: "+maxVar);
				maxValue = maxVar;
				break;
		}
	}
	
	/**
	 * Sets parameters for using custom tree root
	 * @param customRoot
	 * @param rootNr
	 */
	public void setRootParameters(String zoomOption, int rootNr, ArrayList<Integer> zoomNodes) {
		this.zoomOption = zoomOption;
		this.zoomNodes = new ArrayList<>();
		for(PrimeLogNode n:basis.getNodes()){
			if(n.getK() == rootNr){
				this.cRootNr = rootNr;
			}
			for(Integer i:zoomNodes){
				if(n.getK() == i){
					this.zoomNodes.add(i);
				}
			}
		}
		//System.out.println(this.zoomNodes.toString());
	}
	
	
	//Mouse events

	public void mouseClicked(MouseEvent arg0) {
		
	}

	public void mouseEntered(MouseEvent arg0) {
		
	}

	public void mouseExited(MouseEvent arg0) {
		
	}

	public void mousePressed(MouseEvent me) {
		int xClicked = me.getX();
		int yClicked = me.getY();
		for(Integer i: nodeMap.keySet()){
			Point ip = nodeMap.get(i);
			if(xClicked >= ip.getX() && xClicked <= ip.getX()+nodeWidth
		    && yClicked >= ip.getY() && yClicked <= ip.getY()+nodeHeight){
				//System.out.println("Node "+i+" clicked!");
				basis.expandNode(i);
				super.repaint();
			}
		}
	}

	public void mouseReleased(MouseEvent arg0) {
		
	}

	
}
