package org.processmining.models.queueminer.queueclusterer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.deckfour.xes.model.XLog;
import org.processmining.framework.plugin.Progress;
import org.processmining.models.queueminer.matrixfinder.ResActMatrix;
import org.processmining.models.queueminer.utils.Queue;

public class ClusterFinder {
	
	/**
	 * Finds queue clustering and finds arrival rates and work times
	 * @param log
	 * @param ram
	 * @param dc
	 * @return
	 */
	public static QueueClustering findQueueClustering(XLog log, ResActMatrix ram, Progress p){
		QueueClustering qc = greedyCluster(ram,p);
		//for(Queue q:qc.queues){
			//System.out.println("acts: "+q.getActivities().toString());
			//System.out.println("res: "+q.getResources().toString());
			//q.arrivalRate = findAvgArrivalRate(log,dc,q);
			//ArrayList<Long> worktimes = findWorkTimes(log,dc,q);
			//System.out.println("worktimes size: "+worktimes.size());
			//q.meanWork = findAvgMeanWorkTime(worktimes);
			//q.stdWork = findAvgStdWorkTime(worktimes,q.meanWork);
			//System.out.println("queue stats");
			//System.out.println("arrival rate: "+q.arrivalRate);
			//System.out.println("mean serving: "+q.meanWork);
			//System.out.println("std serving: "+q.stdWork);
		//}
		return qc;
	}
	
	public static QueueClustering queuePerAct(XLog log, ResActMatrix ram){
		String[] actnames = ram.actNames;
		HashMap<String,Integer> indActNames = new HashMap<>();
		for(int i = 0; i < actnames.length; i++){
			indActNames.put(actnames[i],i);
		}
		String[] resnames = ram.resNames;
		HashMap<Integer,String> indResNames = new HashMap<>();
		for(int i = 0; i < resnames.length; i++){
			indResNames.put(i,resnames[i]);
		}
		int[][] M = ram.matrix;
		
		ArrayList<Queue> qc = new ArrayList<>();
		for(String act:actnames){
			ArrayList<String> nextActs = new ArrayList<>();
			nextActs.add(act);
			//find resources above greedy threshold
			ArrayList<String> nextRes = new ArrayList<>();
			int actIndex = indActNames.get(act);
			int thressum = 0;
			int nrThres = 0;
			for(int i = 0; i < M[actIndex].length; i++){
				thressum += M[actIndex][i];
				if(M[actIndex][i] > 0){
					nrThres++;
				}
			}
			int threshold = (int) ((thressum/(double)nrThres) * 0.75);
			for(int i = 0; i < M[actIndex].length; i++){
				if(M[actIndex][i] > threshold){
					nextRes.add(indResNames.get(i));
				}
			}
			Queue nextQ = new Queue(nextRes,nextActs);
			qc.add(nextQ);
		}
		
		QueueClustering qcc = new QueueClustering(qc);
		//System.out.println("actname test: "+qcc.queues.get(0).activities.toString());
		return qcc;
	}
	
	private static QueueClustering greedyCluster(ResActMatrix ram, Progress p){
		//Main idea: choose the set of resources for an activity that would give the maximum local score
		//Then: combine activities with overlapping resources to find the best matches
		//In that stage resources can still be removed from the activities
		//Matrix[activity][resource]
		String[] actnames = ram.actNames;
		ArrayList<IndexedString> indActNames = new ArrayList<>();
		for(int i = 0; i < actnames.length; i++){
			indActNames.add(new IndexedString(i,actnames[i],"act"));
		}
		String[] resnames = ram.resNames;
		ArrayList<IndexedString> indResNames = new ArrayList<>();
		for(int i = 0; i < resnames.length; i++){
			indResNames.add(new IndexedString(i,resnames[i],"res"));
		}
		int[][] M = ram.matrix;
		HashMap<Integer,ArrayList<Integer>> singleActClusters= new HashMap<>();
		//For each activity, find the best corresponding resources
		for(int i = 0; i < M.length; i++){
			//Which means: add all those that are above the threshold for the activity
			ArrayList<Integer> resIds = new ArrayList<>();
			int threshold = findThreshold(M,i);;
			for(int j = 0; j < M[i].length; j++){
				if(M[i][j] >= threshold){
					resIds.add(j);
				}
			}
			singleActClusters.put(i,resIds);
		}
		/*System.out.println("Single act clusters:");
		for(Integer key:singleActClusters.keySet()){
			System.out.println("act: "+indActNames.get(key).text);
			ArrayList<Integer> resnrs = singleActClusters.get(key);
			System.out.println("res:");
			for(Integer i:resnrs){
				System.out.println(indResNames.get(i).text);
			}
			System.out.println("---");
		}*/
		//Then try to merge activities brute force
		//Pick the next activity, find the best merge (or no more merge). 
		//If no more merge, go to the next acts
		//If merged, repeat until no more merge
		ArrayList<Queue> qc = new ArrayList<>();
		ArrayList<Integer> actIds = new ArrayList<>();
		ArrayList<Integer> resIds = new ArrayList<>();
		for(int i = 0; i < M.length; i++){
			actIds.add(i);
		}
		
		p.setMinimum(0);
		int maxActIds = actIds.size();
		p.setMaximum(maxActIds);
		
		ArrayList<Integer> currentQ = new ArrayList<>();
		ArrayList<Integer> currentQ2 = new ArrayList<>();
		Integer nextMerge = null;
		boolean first = true;
		while(!actIds.isEmpty()){
			p.setValue(maxActIds-actIds.size());
			//System.out.println("ActIds size: "+actIds.size());
			if(nextMerge == null){
				if(first){
					first = false;
					currentQ = new ArrayList<>();
					currentQ.add(actIds.get(0));
				}
				else{
					//Current queue is found, proceed with next
					ArrayList<String> nextActs = new ArrayList<>();
					ArrayList<String> nextRes = new ArrayList<>();
					for(Integer act:currentQ){
						actIds.remove(act);
						nextActs.add(actnames[act]);
						//System.out.println("nextAct: "+actnames[act]);
						
						ArrayList<Integer> ress = singleActClusters.get(act);
						for(Integer res:ress){
							//System.out.println("nextRes: "+resnames[res]);
							if(!nextRes.contains(resnames[res])){
								nextRes.add(resnames[res]);
							}
						}
					}
					//System.out.println("nextAct size: "+nextActs.size());
					//System.out.println("nextRes zie: "+nextRes.size());
					Queue nextQ = new Queue(nextRes,nextActs);
					qc.add(nextQ);
					//System.out.println("added queue: "+nextActs.toString()+", "+nextRes.toString());
					if(actIds.isEmpty()){
						break;
					}
					currentQ = new ArrayList<>();
					currentQ.add(actIds.get(0));
					//System.out.println("new queue with start: "+currentQ.toString());
				}
			}
			else{
				//merge happened. check for more merges
				actIds.remove(nextMerge);
				currentQ.add(nextMerge);
			}
			nextMerge = null;
			//calculate the scores for current possibilities
			for(Integer act:actIds){
				resIds = new ArrayList<>();
				//find cost of not merging
				for(Integer act2:currentQ){
					ArrayList<Integer> ress = singleActClusters.get(act2);
					for(Integer res:ress){
						if(!resIds.contains(res)){
							resIds.add(res);
						}
					}
				}
				int nonMergeCost = clusterCost(resIds,currentQ,M);
				//System.out.println("new act try: "+act+" with non merge cost: "+nonMergeCost);
				//then compare with all merges
				currentQ2 = new ArrayList<>(currentQ);
				currentQ2.add(act);
				resIds = new ArrayList<>();
				for(Integer act2:currentQ2){
					ArrayList<Integer> ress = singleActClusters.get(act2);
					for(Integer res:ress){
						if(!resIds.contains(res)){
								resIds.add(res);
						}
					}
				}
				int cost = clusterCost(resIds,currentQ2,M);
				if(cost < nonMergeCost){
					nextMerge = act;
					nonMergeCost = cost;
					//System.out.println("Found a cheaper merge: "+currentQ2.toString());
				}
				//System.out.println("(non) merge cost: "+nonMergeCost+" act: "+nextMerge);
			}
		}
		QueueClustering qcc = new QueueClustering(qc);
		//System.out.println("actname test: "+qcc.queues.get(0).activities.toString());
		return qcc;
	}
	
	/*private static QueueClustering  bruteCluster(ResActMatrix ram){
		String[] actnames = ram.actNames;
		ArrayList<IndexedString> indActNames = new ArrayList<>();
		for(int i = 0; i < actnames.length; i++){
			indActNames.add(new IndexedString(i,actnames[i],"act"));
		}
		String[] resnames = ram.resNames;
		ArrayList<IndexedString> indResNames = new ArrayList<>();
		for(int i = 0; i < actnames.length; i++){
			indResNames.add(new IndexedString(i,resnames[i],"res"));
		}
		int[][] M = ram.matrix;
		ArrayList<Queue> qc = new ArrayList<>();
		//Matrix[activity][resource]
		//try all configurations for which holds (for an activity):
		//- only pick resources above the threshold
		//- 
		while(!indActNames.isEmpty()){
			//pick a new starting activity
			IndexedString startAct = indActNames.remove(0);
			//find a powerset of all possibilities that are left
			ArrayList<IndexedString> completeList = new ArrayList<>();
			for(IndexedString r:indResNames){
				completeList.add(r);
			}
			for(IndexedString a:indActNames){
				completeList.add(a);
			}
			Set<IndexedString> set = new HashSet<IndexedString>(completeList);
			Set<Set<IndexedString>> powerset = powerSet(set);
			//find the combination with the least cost
			int minCost = Integer.MAX_VALUE;
			Set<IndexedString> minCombo = new HashSet<>();
			for(Set<IndexedString> possibleComb:powerset){
				boolean hasOneResource = false;
				for(IndexedString is:possibleComb){
					if(is.type.equals("res")){
						hasOneResource = true;
						break;
					}
				}
				//if at least one resource is in the combination
				if(hasOneResource){
					ArrayList<Integer> curRes = new ArrayList<>();
					ArrayList<Integer> curAct = new ArrayList<>();
					curAct.add(startAct.index);
					for(IndexedString is: possibleComb){
						if(is.type.equals("res")){
							curRes.add(is.index);
						}
						else{
							curAct.add(is.index);
						}
					}
					int[] resIds = new int[curRes.size()];
					int[] actIds = new int[curAct.size()];
					for(int i = 0; i < curRes.size(); i++){
						resIds[i] = curRes.get(i);
					}
					for(int i = 0; i < curAct.size(); i++){
						actIds[i] = curAct.get(i);
					}
					int cost = clusterCost(resIds,actIds,M);
					if(cost < minCost){
						minCombo = possibleComb;
						minCost = cost;
					}
				}
			}
			//Add mincombo to queue list and remove mincombo from to be done
			ArrayList<String> resources = new ArrayList<>();
			ArrayList<String> activities = new ArrayList<>();
			activities.add(startAct.text);
			for(IndexedString is:minCombo){
				if(is.type.equals("res")){
					resources.add(is.text);
					indResNames.remove(is);
				}
				else{
					activities.add(is.text);
					indActNames.remove(is);
				}
			}
			Queue q = new Queue(resources,activities);
			qc.add(q);
		}
		
		return new QueueClustering(qc);
	}
	
	/**
	 * Finds powerset given a set
	 * @param originalSet
	 * @return
	 */
	public static <T> Set<Set<T>> powerSet(Set<T> originalSet) {
	    Set<Set<T>> sets = new HashSet<Set<T>>();
	    if (originalSet.isEmpty()) {
	    	sets.add(new HashSet<T>());
	    	return sets;
	    }
	    List<T> list = new ArrayList<T>(originalSet);
	    T head = list.get(0);
	    Set<T> rest = new HashSet<T>(list.subList(1, list.size())); 
	    for (Set<T> set : powerSet(rest)) {
	    	Set<T> newSet = new HashSet<T>();
	    	newSet.add(head);
	    	newSet.addAll(set);
	    	sets.add(newSet);
	    	sets.add(set);
	    }		
	    return sets;
	}
	
	private static int findThreshold(int[][] matrix, int actId){
		//find how many times act is executed
		int totalUsed = 0;
		int resAboveZ = 0;
		for(int j = 0; j < matrix[0].length; j++){
			if(matrix[actId][j] > 0){
				totalUsed += matrix[actId][j];
				resAboveZ++;
			}
		}
		int threshold;
		if(resAboveZ == 0){
			threshold = 0;
		}
		else{
			threshold = (int) ((totalUsed/resAboveZ) * 0.9) ;
		}
		return threshold;
	}
	
	/**
	 * 
	 * @param res
	 * @param act 
	 * @param matrix cost matrix
	 * @return
	 */
	private static int clusterCost(ArrayList<Integer> res, ArrayList<Integer> act, int[][] matrix){
		//matrix of shape M[act][res]
		int cost = 0;
		boolean isIn;
		//resoure costs
		for(int i = 0; i < res.size(); i++){
			//find how many times resource does something
			int totalUsed = 0;
			int actsAboveZ = 0;
			for(int j = 0; j < matrix.length; j++){
				if(matrix[j][res.get(i)] > 0){
					totalUsed += matrix[j][res.get(i)];
					actsAboveZ++;
				}
			}
			int threshold;
			if(actsAboveZ == 0){
				threshold = 0;
			}
			else{
				threshold = (int) (totalUsed/actsAboveZ * 0.75);
			}
			//find costs based on fraction of work
			for(int j = 0; j < matrix.length;j++){
				isIn = false;
				for(int k = 0; k < act.size(); k++){
					if(act.get(k) == j){
						isIn = true;
					}
				}
				if(isIn && matrix[j][res.get(i)] < threshold){
					cost+= threshold - matrix[j][res.get(i)];
				}
				else if (!isIn && matrix[j][res.get(i)] >= threshold){
					cost+= matrix[j][res.get(i)] - threshold;
				}
			}
		}
		//activity costs
		for(int i = 0; i < act.size(); i++){
			//find how many times act is executed
			int totalUsed = 0;
			int actsAboveZ = 0;
			for(int j = 0; j < matrix[0].length; j++){
				if(matrix[act.get(i)][j] > 0){
					totalUsed += matrix[act.get(i)][j];
					actsAboveZ++;
				}
			}
			int threshold;
			if(actsAboveZ == 0){
				threshold = 0;
			}
			else{
				threshold = (int) (totalUsed/actsAboveZ* 0.75);
			}
			//find costs based on fraction of work
			for(int j = 0; j < matrix[act.get(i)].length;j++){
				isIn = false;
				for(int k = 0; k < res.size(); k++){
					if(res.get(k) == j){
						isIn = true;
					}
				}
				if(isIn && matrix[act.get(i)][j] < threshold){
					cost+= threshold - matrix[act.get(i)][j];
				}
				else if (!isIn && matrix[act.get(i)][j] >= threshold){
					cost+= matrix[act.get(i)][j] - threshold;
				}
			}
		}
		return cost;
	}
}
