package org.processmining.models.queueminer.queuepredictor;

import java.util.ArrayList;

import org.processmining.models.queueminer.distribution.Point;
import org.processmining.models.queueminer.queueclusterer.QueueClustering;
import org.processmining.models.queueminer.utils.ArrivingEvent;
import org.processmining.models.queueminer.utils.Queue;

public class QueuePredictor {

	/**
	 * given an incoming event at some time, predict the time the event is done
	 * @param qc
	 * @param ae
	 * @param timeWaited
	 * @return
	 */
	public static void predictQueueExitTime(QueueClustering qc, ArrivingEvent ae, long timeWaited){
		String eventType = ae.eventType;
		//Find the queue this activity belongs to
		//System.out.println("queues: "+qc.getQueues().toString());
		
		Queue rightQ = qc.getQueues().get(0);
		for(Queue q:qc.getQueues()){
			if(q.getActivities().contains(eventType)){
				rightQ = q;
			}
		}
		//find the distribution corresponding to context and time
		long time = ae.arrivalTime;
		long weekTime = time % (7 * 24 * 3600 * 1000);
		long subTime = rightQ.getSubTime();
		int k = (int) (weekTime/subTime);
		//System.out.println("weekTime: "+weekTime+", k: "+k);
		String context = ae.context;
		long mostLikelyK;
		
		if(rightQ.getServiceRate() == null || rightQ.getServiceRate().isEmpty() ){
			//System.out.println("mlk 0 because no entry found in map");
			mostLikelyK = 0;
		}
		else if(rightQ.getServiceRate().get(context) == null
				|| rightQ.getServiceRate().get(context).isEmpty()){
			//get first context and use k if possible
			for(String key:rightQ.getServiceRate().keySet()){
				context = key;
				break;
			}
			//if k cant be used, use first k
			if(rightQ.getServiceRate().get(context).get(k) == null 
					|| rightQ.getServiceRate().get(context).get(k).isEmpty() ){
				//get first k instead
				for(Integer key: rightQ.getServiceRate().get(context).keySet()){
					k = key;
					break;
				}
			}
			
		}
		else if(rightQ.getServiceRate().get(context).get(k) == null 
				|| rightQ.getServiceRate().get(context).get(k).isEmpty() ){
			//System.out.println("Took closest key since k wasnt found");
			//get closest k instead
			int closestK = 0;
			for(Integer key: rightQ.getServiceRate().get(context).keySet()){
				if(Math.abs(k-key) < Math.abs(k-closestK)){
					closestK = key;
				}
			}
			k = closestK;
		}
		ArrayList<Point> dist = rightQ.getServiceRate().get(context).get(k);
		//find the highest probability time given the time waited
		int waitK = (int) (timeWaited/subTime);
		mostLikelyK = waitK;
		long maxAmount = 0;
		if(dist == null){
			mostLikelyK = 0;
		}
		else{
			for(Point p:dist){
				if(p.getX() >= waitK && p.getY() > maxAmount){
					maxAmount = p.getY();
					mostLikelyK = p.getX()/subTime;
				}
			}
		}
		/*System.out.println("most likely k: "+mostLikelyK);
		System.out.println("time: "+time);
		System.out.println("subtime: "+subTime);
		System.out.println("totalPredTime: "+(time + mostLikelyK * subTime));*/
		//convert back to actual time
		//System.out.println("mostLikelyk: "+mostLikelyK);
		ae.predictTime(time + mostLikelyK * subTime);
	}
	
	
}
