package org.processmining.models.queueminer.queueratefinder;

import java.net.URI;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;

import org.deckfour.xes.extension.XExtension;
import org.deckfour.xes.extension.XExtensionManager;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.framework.plugin.Progress;
import org.processmining.models.queueminer.distribution.Point;
import org.processmining.models.queueminer.patternvisualizer.GranularityFindingUtils;
import org.processmining.models.queueminer.queueclusterer.QueueClustering;
import org.processmining.models.queueminer.utils.Queue;
import org.processmining.models.queueminer.utils.TimeActivity;
import org.processmining.models.queueminer.utils.TimeActivityUtilities;

public class QueueRateFinder {
	
	public static QueueClustering findQueueClustering(XLog log, QueueClustering qc, int k, Progress p){
		//find the number of unique contexts
		if(log.size() == 0){
			QueueClustering emptyResult = new QueueClustering(new ArrayList<Queue>());
			emptyResult.setK(k);
			return emptyResult;
		}
		
		int nrWeeks = findNrWeeksSpanned(log);
		int weekParts = k * 7;
		long weekSize = 7 * 24 * 3600 * 1000;
		long subTime = (long) ((weekSize)/(double)weekParts);
		
		//Split up events based on queue, context and k
		HashMap<Queue,HashMap<String,HashMap<Integer,ArrayList<TimeActivity>>>> splitUpEvents = new HashMap<>();
		XTimeExtension xte = XTimeExtension.instance();
		XExtensionManager xem = XExtensionManager.instance();
		XConceptExtension xcone = XConceptExtension.instance();
		URI contextUri = URI.create("http://www.zerodivision.nl/XES/context.xesext");
		XExtension xce = xem.getByUri(contextUri);
		Collection<XAttribute> attributes = xce.getEventAttributes();
		XAttribute contattr = attributes.iterator().next();
		
		p.setCaption("Split up log depending on context and time interval");
		p.setMinimum(0);
		p.setMaximum(log.size());
		
		int progc = 0;
		for(XTrace tr:log){
			ArrayList<XEvent> traceEvents = new ArrayList<>();
			for(XEvent ev:tr){
				traceEvents.add(ev);
			}
			if(traceEvents.size() > 1){
				for(int i = 0; i < traceEvents.size()-1; i++){
					XEvent next = traceEvents.get(i+1);
					XEvent ev = traceEvents.get(i);
					String context = "Regular";
					String name = xcone.extractName(ev);
					long time = xte.extractTimestamp(ev).getTime();
					long timeInWeek = time % weekSize;
					long nextime = xte.extractTimestamp(next).getTime();
					int curk = getKFromSubTime(timeInWeek,subTime);
					Queue q = findQueue(name,qc);	
					try{
						context = ev.getAttributes().get(contattr.getKey()).toString();
					}
					catch(Exception e){}
					
					//new maps/lists if necessary
					if(!splitUpEvents.containsKey(q)){
						splitUpEvents.put(q, new HashMap<String,HashMap<Integer,ArrayList<TimeActivity>>>());
					}
					if(!splitUpEvents.get(q).containsKey(context)){
						splitUpEvents.get(q).put(context, new HashMap<Integer,ArrayList<TimeActivity>>());
					}
					if(!splitUpEvents.get(q).get(context).containsKey(curk)){
						splitUpEvents.get(q).get(context).put(curk, new ArrayList<TimeActivity>());
					}
					//add new event
					splitUpEvents.get(q).get(context).get(curk).add(new TimeActivity(time,nextime,name,ev));

				}
			}
			p.setValue(progc);
			progc++;
			/**/
		}
		
		p.setCaption("Finding arrival rates and service distributions for each log part");
		p.setMinimum(0);
		p.setMaximum(splitUpEvents.keySet().size());
		
		progc = 0;
		//find distributions for each entry in the map
		//at the same time, find the amount of events that arrived at that bin (count dist)
		long distGranularity = subTime; //Let's be quite precise in all cases
		//long distGranularity = 3600*1000;//1 hour
		HashMap<Queue,HashMap<String,HashMap<Integer,ArrayList<Point>>>> splitUpDistributions = new HashMap<>();
		HashMap<Queue,HashMap<String,HashMap<Integer,Integer>>> splitUpArrivalRates = new HashMap<>();
		for(Queue q:splitUpEvents.keySet()){
			HashMap<String,HashMap<Integer,ArrayList<Point>>> ckdist = new HashMap<>();
			HashMap<String,HashMap<Integer,Integer>> ckarr = new HashMap<>();
			for(String context:splitUpEvents.get(q).keySet()){
				HashMap<Integer,Integer> karr = new HashMap<>();
				HashMap<Integer,ArrayList<Point>> kdist = new HashMap<>();
				for(Integer kpart:splitUpEvents.get(q).get(context).keySet()){
					ArrayList<TimeActivity> events = splitUpEvents.get(q).get(context).get(kpart);
					ArrayList<Point> dist = TimeActivityUtilities.graphActivityList(events, distGranularity);
					if(dist.size() > 0){
						//System.out.println("k:"+kpart);
						//System.out.println("dist : "+dist.toString());
					}
					kdist.put(kpart, dist);
					int distSize = GranularityFindingUtils.findPointListSize(dist);
					karr.put(kpart, distSize/nrWeeks);
					//System.out.println("Distribution size: "+dist.size());
				}
				ckdist.put(context, kdist);
				ckarr.put(context, karr);
			}
			splitUpDistributions.put(q, ckdist);
			splitUpArrivalRates.put(q, ckarr);
			p.setValue(progc);
			progc++;
		}
		
		//Create queue clustering of resulting distributions
		ArrayList<Queue> resultQueues = new ArrayList<>();
		for(Queue q:splitUpDistributions.keySet()){
			q.setQueueServiceRate(splitUpDistributions.get(q));
			q.setQueueArrivalRate(splitUpArrivalRates.get(q), subTime);
			resultQueues.add(q);
		}
		QueueClustering newQC = new QueueClustering(resultQueues);
		newQC.setK(k);
		return  newQC;
	}
	
	public static int getKFromSubTime(long arrivalTime, long subTime){
		return (int) (arrivalTime/subTime);
	}
	
	public static Queue findQueue(String name, QueueClustering qc){
		for(Queue q:qc.getQueues()){
			if(q.getActivities().contains(name)){
				return q;
			}
		}
		return null;
	}
	
	public static int findNrWeeksSpanned(XLog log){
		ArrayList<TimeActivity> tlog = TimeActivityUtilities.findTimeActivities(log);
		Collections.sort(tlog);
		//System.out.println("tlog size: "+tlog.size());
		long minTime = tlog.get(0).start;
		long maxTime = tlog.get(tlog.size()-1).start;
		long weekSize = 7 * 24 * 3600 * 1000;
		return (int) ((maxTime-minTime)/weekSize);
	}
	
	/**
	 * Returns a list of all unique context strings in the log
	 * @param log
	 * @return
	 */
	public static ArrayList<String> findContexts(XLog log){
		ArrayList<String> contexts = new ArrayList<>();
		XExtensionManager xem = XExtensionManager.instance();
		URI conceptUri = URI.create("http://www.zerodivision.nl/XES/context.xesext");
		XExtension xce = xem.getByUri(conceptUri);
		Collection<XAttribute> attributes = xce.getEventAttributes();
		XAttribute contattr = attributes.iterator().next();
		String key = contattr.getKey();
		
		for(XTrace tr:log){
			for(XEvent ev:tr){
				String context = "Regular";
				try{
					context = ev.getAttributes().get(key).toString();
				}
				catch(Exception e){}
				if(!contexts.contains(context)){
					//System.out.println("adding : "+context);
					contexts.add(context);
				}
			}
		}
		
		return contexts;
	}
}
