package org.processmining.models.queueminer.sojournpredictor;

import java.util.ArrayList;

import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.queueminer.context.ConcatenatedContext;
import org.processmining.models.queueminer.logcontext.LogWorkloadContextFinder;
import org.processmining.models.queueminer.queueclusterer.QueueClustering;
import org.processmining.models.queueminer.queuepredictor.QueuePredictor;
import org.processmining.models.queueminer.utils.ArrivingEvent;

public class QueueSojournPredictor {
	TemporalRouter tr;
	QueueClustering qc;
	XLog log;
	int k;
	ConcatenatedContext cc;
	
	public QueueSojournPredictor(XLog training, int k, QueueClustering qc){
		tr = new TemporalRouter(training,k);
		cc = LogWorkloadContextFinder.findWorkLoadInformation(training).concatenateAll();
		this.qc = qc;
		this.log = training;
		this.k = k;
	}
	
	public long predictSojournTime(ArrivingEvent ae, boolean useContext){
		ArrayList<String> prevs = new ArrayList<>();
		int longestTrSize = findLongestTraceSize();
		long exitTime = ae.arrivalTime;
		int weekParts = k * 7;
		long weekSize = 7 * 24 * 3600 * 1000;
		long subTime = (long) ((weekSize)/(double)weekParts);
		int i = 0;
		//System.out.println("arriving event type: "+ae.eventType);
		//System.out.println("predicted route: ");
		//System.out.print("Acts:");
		while(!ae.eventType.equals("__END__") && i <= longestTrSize){
			//find the time it takes to exit the queue
			QueuePredictor.predictQueueExitTime(qc, ae, 0);
			exitTime = ae.getPredictedTime();
			//find the context at the exit time
			String context;
			//if context variables are being used, predict the right context
			if(useContext){
				context = cc.getContextAtTime(exitTime);
			}
			else{//otherwise just predict regular
				context = "Regular";
			}
			
			long timeInWeek = exitTime % weekSize;
			int nextk = (int) (timeInWeek/subTime);
			//find the next activity and update the arriving event
			prevs.add(ae.eventType);
			String next = tr.predictNextActivity(prevs, context,nextk);
			//System.out.print(next+", ");
			//System.out.println(next+" ");
			ae = new ArrivingEvent(next,exitTime,context,"");
			ae.setDoneTime(0);//done time is irrelevant here
			i++;
		}
		//System.out.println();
		return exitTime;
	}
	
	/**
	 * Returns the size of the longest trace in the log
	 * @return
	 */
	public int findLongestTraceSize(){
		int max = 0;
		for(XTrace tr:log){
			if(tr.size() > max){
				max = tr.size();
			}
		}
		return max;
	}
}
