package org.processmining.models.queueminer.sojournpredictor;

import java.net.URI;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;

import org.deckfour.xes.extension.XExtension;
import org.deckfour.xes.extension.XExtensionManager;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;

public class TemporalRouter {
	XLog log;
	int k;
	HashMap<String, HashMap<String, HashMap<Integer, String>>> routerInfo;
	//format: >previous events->context->time->value
	HashMap<String, HashMap<String, HashMap<Integer, String>>> simpleInfo;

	//format: previous event->context->time->value

	public TemporalRouter(XLog log, int k) {
		this.log = log;
		this.k = k;
		trainRouter();
	}

	private void trainRouter() {
		setRouterInfo();
		//setSimpleInfo();
	}

	private void setRouterInfo() {
		XConceptExtension xce = XConceptExtension.instance();
		URI contextUri = URI.create("http://www.zerodivision.nl/XES/context.xesext");
		//Context/time handling
		XExtensionManager xem = XExtensionManager.instance();
		XExtension xcoe = xem.getByUri(contextUri);
		Collection<XAttribute> attributes = xcoe.getEventAttributes();
		XAttribute contattr = attributes.iterator().next();
		int weekParts = k * 7;
		long weekSize = 7 * 24 * 3600 * 1000;
		long subTime = (long) ((weekSize) / (double) weekParts);
		XTimeExtension xte = XTimeExtension.instance();
		//Format: countedNext<previous event,<context,<k,<next event, amount>>>>
		HashMap<String, HashMap<String, HashMap<Integer, HashMap<String, EventCount>>>> countedNext = new HashMap<>();
		//make map of queue/context/k/prefix data
		ArrayList<String> prefix;
		
		for (XTrace tr : log) {
			String lastContext = "Regular";
			Integer lastK = 0;
			prefix = new ArrayList<>();
			for (XEvent ev : tr) {
				//given the prefix, given the exit time, given the context, add the next prediction value count
				String evName = xce.extractName(ev);
				if(evName == null){
					evName = "";
				}
				if (!prefix.isEmpty()) {
					countedNext = insertEvent(countedNext, prefix.toString(), lastContext, lastK, evName);
				}
				prefix.add(evName);
				Collections.sort(prefix);
				//Handle time and context info
				try {
					lastContext = ev.getAttributes().get(contattr.getKey()).toString();
				} catch (Exception e) {}
				long time = xte.extractTimestamp(ev).getTime();
				long timeInWeek = time % weekSize;
				lastK = (int) (timeInWeek / subTime);
			}
			countedNext = insertEvent(countedNext, prefix.toString(), lastContext, lastK, "__END__");
		}

		//find most likely next event based on count maps
		routerInfo = new HashMap<>();
		for (String prev : countedNext.keySet()) {
			for (String cont : countedNext.get(prev).keySet()) {
				for (Integer k : countedNext.get(prev).get(cont).keySet()) {
					//find the most likely event for context cont and k given the previous event is prev
					int max = 0;
					String maxVal = "__END__";
					for (String next : countedNext.get(prev).get(cont).get(k).keySet()) {
						EventCount evc = countedNext.get(prev).get(cont).get(k).get(next);
						if (evc.count > max) {
							max = evc.count;
							maxVal = evc.type;
						}
					}
					//put the most likely next event in the map
					if (!routerInfo.containsKey(prev)) {
						routerInfo.put(prev, new HashMap<String, HashMap<Integer, String>>());
					}
					if (!routerInfo.get(prev).containsKey(cont)) {
						routerInfo.get(prev).put(cont, new HashMap<Integer, String>());
					}
					routerInfo.get(prev).get(cont).put(k, maxVal);
				}
			}
		}
	}

	/**
	 * Insert/Create map for keeping track of count for every next event for any
	 * given event in some context/time
	 * 
	 * @param countedNext
	 * @param last
	 * @param lastContext
	 * @param lastK
	 * @param evName
	 */
	private HashMap<String, HashMap<String, HashMap<Integer, HashMap<String, EventCount>>>> insertEvent(
			HashMap<String, HashMap<String, HashMap<Integer, HashMap<String, EventCount>>>> countedNext,
			String prefix, String lastContext, Integer lastK, String evName) {
		
		if (!countedNext.containsKey(prefix)) {
			countedNext.put(prefix, new HashMap<String, HashMap<Integer, HashMap<String, EventCount>>>());
		}
		if (!countedNext.get(prefix).containsKey(lastContext)) {
			countedNext.get(prefix).put(lastContext, new HashMap<Integer, HashMap<String, EventCount>>());
		}
		if (!countedNext.get(prefix).get(lastContext).containsKey(lastK)) {
			countedNext.get(prefix).get(lastContext).put(lastK, new HashMap<String, EventCount>());
		}
		if (!countedNext.get(prefix).get(lastContext).get(lastK).containsKey(evName)) {
			countedNext.get(prefix).get(lastContext).get(lastK).put(evName, new EventCount(evName));
		} else {
			countedNext.get(prefix).get(lastContext).get(lastK).get(evName).count++;
		}
		
		return countedNext;
	}

	/**
	 * Special return string: "__END__" if no next entry
	 * 
	 * @return
	 */
	public String predictNextActivity(ArrayList<String> previousActs, String context, Integer k) {
		Collections.sort(previousActs);
		//String lastAct = previousActs.get(previousActs.size()-1);
		String prefix = previousActs.toString();
		if (routerInfo.containsKey(prefix)) {
			if (routerInfo.get(prefix).containsKey(context)) {
				if (routerInfo.get(prefix).get(context).containsKey(k)) {
					return routerInfo.get(prefix).get(context).get(k);
				}
			}
		}
		//Simple info is not as good an indication as bag
		/*
		 * if(simpleInfo.containsKey(lastAct)){
		 * if(simpleInfo.get(lastAct).containsKey(context)){
		 * if(simpleInfo.get(lastAct).get(context).containsKey(k)){ return
		 * simpleInfo.get(lastAct).get(context).get(k); } } }
		 */
		return "__END__";
	}

}
