package org.processmining.models.queueminer.utils;

import java.net.URI;
import java.util.Collection;

import org.deckfour.xes.extension.XExtension;
import org.deckfour.xes.extension.XExtensionManager;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;

public class ArrivingEvent implements Comparable<ArrivingEvent>{
	public String eventType;
	public long arrivalTime;
	public String context;
	private long doneTime;
	private long predictedTime;
	private String identifier;
	
	/**
	 * 
	 * @param type
	 * @param arrivalTime
	 * @param context
	 * @param doneTime
	 */
	public ArrivingEvent(String type, long arrivalTime, String context, String identifier){
		eventType = type;
		this.arrivalTime = arrivalTime;
		this.context = context;
		this.identifier = identifier;
	}
	
	public void setDoneTime(long doneTime){
		this.doneTime = doneTime;
	}
	
	public void predictTime(long time){
		this.predictedTime = time;
	}
	
	public long getPredictedTime(){
		return predictedTime;
	}
	
	public long getPredictionError(){
		return Math.abs(predictedTime-doneTime);
	}
	
	public long getSquaredError(){
		return (predictedTime-doneTime)*(predictedTime-doneTime);
	}
	
	public long getDoneTime(){
		return doneTime;
	}
	
	public String getID(){
		return identifier;
	}
	
	public static ArrivingEvent fromLogEvent(XEvent thisEvent){
		XConceptExtension xce = XConceptExtension.instance();
		String type = xce.extractName(thisEvent);
		XTimeExtension xte = XTimeExtension.instance();
		long arrivalTime = xte.extractTimestamp(thisEvent).getTime();
		
		URI contextUri = URI.create("http://www.zerodivision.nl/XES/context.xesext");
		XExtensionManager xem = XExtensionManager.instance();
		XExtension xcte = xem.getByUri(contextUri);
		Collection<XAttribute> attributes = xcte.getEventAttributes();
		XAttribute contattr = attributes.iterator().next();
		//use regular context if no context is set
		String context = "Regular";
		try{
			context = thisEvent.getAttributes().get(contattr.getKey()).toString();
		}
		catch(Exception e){}
		
		return new ArrivingEvent(type,arrivalTime,context,thisEvent.getID().toString());
	}
	
	public static ArrivingEvent fromLogEvents(XEvent thisEvent, XEvent nextEvent){
		XTimeExtension xte = XTimeExtension.instance();
		long doneTime = xte.extractTimestamp(nextEvent).getTime();
		ArrivingEvent singleEv = fromLogEvent(thisEvent);
		singleEv.setDoneTime(doneTime);
		
		return singleEv;
	}

	public int compareTo(ArrivingEvent ae) {
		return Long.compare(this.arrivalTime, ae.arrivalTime);
	}
	
	
}
