package org.processmining.models.queueminer.utils;

import java.net.URI;
import java.util.Collection;

import org.deckfour.xes.extension.XExtension;
import org.deckfour.xes.extension.XExtensionManager;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;

public class ArrivingTrace {
	public String eventType;
	public long arrivalTime;
	public String context;
	private long doneTime;
	private long predictedTime;
	private String evIdentifier;
	private String trIdentifier;
	
	/**
	 * 
	 * @param type
	 * @param arrivalTime
	 * @param context
	 * @param doneTime
	 */
	public ArrivingTrace(String type, long arrivalTime, String context, String traceIdentifier, String eventIdentifier){
		eventType = type;
		this.arrivalTime = arrivalTime;
		this.context = context;
		this.evIdentifier = eventIdentifier;
		this.trIdentifier = traceIdentifier;
	}
	
	public void setDoneTime(long doneTime){
		this.doneTime = doneTime;
	}
	
	public void predictTime(long time){
		this.predictedTime = time;
	}
	
	public long getPredictedTime(){
		return predictedTime;
	}
	
	public long getPredictionError(){
		return Math.abs(predictedTime-doneTime);
	}
	
	public long getSquaredError(){
		return (predictedTime-doneTime)*(predictedTime-doneTime);
	}
	
	public long getDoneTime(){
		return doneTime;
	}
	
	public String getEventID(){
		return evIdentifier;
	}
	
	public String getTraceID(){
		return trIdentifier;
	}
	
	public static ArrivingTrace fromLogTrace(XTrace thisTrace){
		//Find the event to be considered: current last event
		XEvent thisEvent = thisTrace.get(thisTrace.size()-1);
		
		XConceptExtension xce = XConceptExtension.instance();
		String type = xce.extractName(thisEvent);
		XTimeExtension xte = XTimeExtension.instance();
		long arrivalTime = xte.extractTimestamp(thisEvent).getTime();
		String traceName = xce.extractName(thisTrace);
		
		URI contextUri = URI.create("http://www.zerodivision.nl/XES/context.xesext");
		XExtensionManager xem = XExtensionManager.instance();
		XExtension xcte = xem.getByUri(contextUri);
		Collection<XAttribute> attributes = xcte.getEventAttributes();
		XAttribute contattr = attributes.iterator().next();
		//use regular context if no context is set
		String context = "Regular";
		try{
			context = thisEvent.getAttributes().get(contattr.getKey()).toString();
		}
		catch(Exception e){}
		
		return new ArrivingTrace(type,arrivalTime,context,traceName,thisEvent.getID().toString());
	}

	public int compareTo(ArrivingEvent ae) {
		return Long.compare(this.arrivalTime, ae.arrivalTime);
	}
}
