package org.processmining.models.queueminer.utils;

import java.math.BigInteger;
import java.util.ArrayList;

import org.processmining.models.queueminer.distribution.Point;

public class StatisticUtil {
	
	public static double[] pointsToDoubleArray(ArrayList<Point> points){
		double[] data = new double[points.size()];
		for(int i = 0; i < points.size(); i++){
			data[i] = points.get(i).getY();
		}
		return data;
	}
	
	public static ArrayList<Long> pointsToLongList(ArrayList<Point> points){
		ArrayList<Long> longs = new ArrayList<>();
		for(Point p:points){
			longs.add(p.getY());
		}
		return longs;
	}
	
	private static long sqr(long value){
		return value * value;
	}
	
	/**
	 * @pre actualVals.size() == predictedVals.size()
	 * @param actualVals
	 * @param predictedVals
	 * @return
	 */
	public static double getNormalizedRootMeanSquare(ArrayList<Long> actualVals, ArrayList<Long> predictedVals){
		assert actualVals.size() == predictedVals.size();
		BigInteger avg = BigInteger.valueOf(actualVals.get(0)-predictedVals.get(0));
		avg = avg.multiply(avg);
		BigInteger sum = avg;
		long min = Long.MAX_VALUE;
		long max = 0;
		//try{
		for(int i = 1; i < actualVals.size(); i++){
			/*if(i*avg.longValue() + sqr(longs.get(i)) < 0){
				throw new Exception("Long overflow in getMean function!");
			}*/
			//avg = avg.multiply(BigInteger.valueOf(i));
			long error = actualVals.get(i) - predictedVals.get(i);
			//System.out.println("error at "+i+": "+error);
			BigInteger newErr = BigInteger.valueOf(error);
			newErr = newErr.multiply(newErr);
			//avg = avg.add(newErr);
			//avg = avg.divide(BigInteger.valueOf(i+1));
			
			sum.add(newErr);
			
			//keep track of max and min
			if(actualVals.get(i) > max){
				max = actualVals.get(i);
			}
			if(actualVals.get(i) < min){
				min = actualVals.get(i);
			}
		}
		avg = sum.divide(BigInteger.valueOf(actualVals.size()));
		//System.out.println("avg before sqrt: "+avg.toString());
		/*}
		catch(Exception e){
			System.out.println(e);
		}*/
		//System.out.println("max : "+max+", min:"+min);
		if(max == min){
			if(max != 0){
				avg = getBigIntRoot(avg);
				double NRMSE = (double)avg.longValue() / (double)max;
				//avg = avg.divide(BigInteger.valueOf(max));
				return NRMSE;
			}
			else{
				return 0;
			}
		}
		else{
			avg = getBigIntRoot(avg);
			//avg = avg.divide(BigInteger.valueOf(max-min));
			double NRMSE = (double)avg.longValue() / (double)(max-min);
			//System.out.println("avg after divide max-min: "+NRMSE);
			return NRMSE;
		}
	}
	
	public static BigInteger getBigIntRoot(BigInteger b){
		if(b.longValue() == 0){
			return BigInteger.ZERO;
		}
		//newton's iteration
		BigInteger two = BigInteger.valueOf(2);
		BigInteger start = b;
		BigInteger lastb = BigInteger.ZERO;
		int c = 0;
		while(b != lastb && c < 10000 && b.longValue() != 0){
			lastb = b;
			BigInteger inter = start;
			inter = inter.divide(b);
			b= b.add(inter);
			b= b.divide(two);
			c++;
		}
		//System.out.println("avg after newtons iteration: "+b.toString());
		return b;
	}
	
	public static long getRootMeanSquare(ArrayList<Long> longs){
		BigInteger avg = BigInteger.valueOf(sqr(longs.get(0)));
		//try{
		for(int i = 1; i < longs.size(); i++){
			/*if(i*avg.longValue() + sqr(longs.get(i)) < 0){
				throw new Exception("Long overflow in getMean function!");
			}*/
			avg = avg.multiply(BigInteger.valueOf(i));
			avg = avg.add(BigInteger.valueOf(sqr(longs.get(i))));
			avg = avg.divide(BigInteger.valueOf(i+1));
		}
		/*}
		catch(Exception e){
			System.out.println(e);
		}*/
		return (long) Math.sqrt(avg.longValue());
	}
	
	public static long getMean(ArrayList<Long> longs){
		BigInteger avg = BigInteger.valueOf(longs.get(0));
		try{
		for(int i = 1; i < longs.size(); i++){
			if(i*avg.longValue() + longs.get(i) < 0){
				throw new Exception("Long overflow in getMean function!");
			}
			avg = BigInteger.valueOf((i*avg.longValue() + longs.get(i))/(i+1));
		}
		}
		catch(Exception e){
			System.out.println(e);
		}
		return avg.longValue();
	}
	
	public static long getStandardDeviation(long meann, ArrayList<Long> longs){
		long var = getVariance(meann,longs);
		//Find the square root
		int k = 1000;	
		long root = 1;
		for(int i = 0; i < k; i++){
			root = (root + var/root)/2;
		}
		return root;		
	}
	
	public static long getVariance(long meann, ArrayList<Long> longs){
		BigInteger mean = BigInteger.valueOf(meann);
		BigInteger first =  BigInteger.valueOf(longs.get(0));
		BigInteger meanDiff = BigInteger.valueOf(((first.longValue()-mean.longValue())*(first.longValue()-mean.longValue())));
		try{
		for(int i = 1; i < longs.size(); i++){
			BigInteger l = BigInteger.valueOf(longs.get(i));
			if(meanDiff.longValue() * i + ((l.longValue()-mean.longValue())*(l.longValue()-mean.longValue())) < 0){
				throw new Exception("Long overflow in getStandardDeviation function!");
			}
			meanDiff = BigInteger.valueOf((meanDiff.longValue() * i + ((l.longValue()-mean.longValue())*(l.longValue()-mean.longValue())))/(i+1));
		}
		}
		catch(Exception e){
			System.out.println(e);
		}
		return meanDiff.longValue();
	}
	
	//Double approach
	
	public static double getMean(double[] data)
    {
		int size = data.length;
        double sum = 0.0;
        for(double a : data)
            sum += a;
            return sum/size;
    }
	
	public static double getMean(int[] data){
		double[] data2 = new double[data.length];
		for(int i = 0; i < data.length; i++){
			data2[i] = data[i];
		}
		return getMean(data2);
	}
	
	public static double getVariance(double[] data)
    {
		int size = data.length;
        double mean = getMean(data);
        double temp = 0;
        for(double a :data)
            temp += (mean-a)*(mean-a);
            return temp/size;
    }

	public static double getStdDev(double[] data)
    {
        return Math.sqrt(getVariance(data));
    }
	
	public static double getStdDev(int[] data){
		double[] data2 = new double[data.length];
		for(int i = 0; i < data.length; i++){
			data2[i] = data[i];
		}
		return getStdDev(data2);
	}
	
	public static double[] cumulativeDist(double[] data){
		//find total size
		double totalSize = 0;
		for(int i = 0; i < data.length; i++){
			totalSize+= data[i];
		}
		//find cumulative distribution
		double[] cumulDist = new double[data.length];
		double lastTotal = 0;
		for(int i = 0;i < data.length; i++){
			lastTotal = lastTotal + data[i];
			cumulDist[i] = lastTotal/totalSize;
		}
		return cumulDist;
	}
	
	public static double findClosestLargerZ(double[] cumulData, double Z){
		for(int i = 0; i < cumulData.length; i++){
			if(cumulData[i] > Z){
				return i;
			}
		}
		return cumulData.length;
	}
	
	public static long[] minMaxValue(ArrayList<Point> points){
		long[] minMax = new long[2];
		for(Point p:points){
			long y = p.getY();
			if(y > minMax[1]){
				minMax[1] = y;
			}
			if(y < minMax[0]){
				minMax[0] = y;
			}
		}
		return minMax;
	}
}
