package org.processmining.plugins.queueminer.bottleneckfinder;

import java.util.ArrayList;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.Connection;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.queueminer.bottleneckfinder.BottleneckData;
import org.processmining.models.queueminer.queueclusterer.QueueClustering;
import org.processmining.models.queueminer.utils.Queue;

@Plugin(name = "Find bottlenecks in queue clustering (All-in-One)", returnLabels = { "Bottleneck data" }, returnTypes = { BottleneckData.class }, parameterLabels = {
		"XLog", "Parameters" }, userAccessible = true)
public class AllInOneBottleNeckFinderPlugin {
	
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "L.P.J. de Smet", email = "l.p.j.d.smet@student.tue.nl")
	@PluginVariant(variantLabel = "Find bottleneck data given queue clustering(All-in-One)", requiredParameterLabels = { 0})
	public BottleneckData findErrorOfPredictionAIO(UIPluginContext context, XLog inputLog) throws Exception {
		QueueClustering qc = new QueueClustering(new ArrayList<Queue>());
		try {
			qc = context.tryToFindOrConstructFirstNamedObject(QueueClustering.class, "Queue rate finder", Connection.class, "", inputLog);
		} catch (ConnectionCannotBeObtained e) {
			e.printStackTrace();
		}
		if(qc.getQueues().isEmpty()){
			return null;
		}
		
		BottleneckData bd = null;
		try {
			bd = context.tryToFindOrConstructFirstNamedObject(BottleneckData.class, "Find bottlenecks in queue clustering", Connection.class, "", qc);
		} catch (ConnectionCannotBeObtained e) {
			e.printStackTrace();
		}		
				
		return bd;
	}
}
