package org.processmining.plugins.queueminer.queuepredictor;

import java.util.ArrayList;

import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.Connection;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.queueminer.queueclusterer.QueueClustering;
import org.processmining.models.queueminer.queuepredictor.PredictionDataContainer;
import org.processmining.models.queueminer.queuepredictor.QueuePredictor;
import org.processmining.models.queueminer.utils.ArrivingEvent;
import org.processmining.models.queueminer.utils.Queue;

@Plugin(name = "Make exit time predictions based on a queueing model (All-in-one)", returnLabels = { "Prediction data" }, returnTypes = { PredictionDataContainer.class }, parameterLabels = {
		"XES Log", "QueueClustering", "Parameters" }, userAccessible = true)
public class AIOQueueExitTimePredictorPlugin {
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "L.P.J. de Smet", email = "l.p.j.d.smet@student.tue.nl")
	@PluginVariant(variantLabel = "Execute queue-based predictions", requiredParameterLabels = { 0,1 })
	public PredictionDataContainer findErrorOfPredictionAIO(UIPluginContext context, XLog trainingData, XLog testingData) throws Exception {
		QueueClustering qc = new QueueClustering(new ArrayList<Queue>());
		try {
			qc = context.tryToFindOrConstructFirstNamedObject(QueueClustering.class, "Queue rate finder", Connection.class, "", trainingData);
		} catch (ConnectionCannotBeObtained e) {
			e.printStackTrace();
		}
		if(qc.getQueues().isEmpty()){
			return null;
		}
		
		//put test data in convenient format
		ArrayList<ArrivingEvent> testValues = new ArrayList<>();
		for(XTrace tr:testingData){
			for(XEvent ev:tr){
				testValues.add(ArrivingEvent.fromLogEvent(ev));
			}
		}
		//Then for all arriving events predict the exit time
		PredictionDataContainer pdc = new PredictionDataContainer();
		for(ArrivingEvent testV:testValues){
			QueuePredictor.predictQueueExitTime(qc, testV, 0);
			pdc.addPredictionData(testV);
		}
		//then return the result
		return pdc;
	}
}
