package org.processmining.plugins.queueminer.queuepredictor;

import java.util.ArrayList;

import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.queueminer.queueclusterer.QueueClustering;
import org.processmining.models.queueminer.queuepredictor.PredictionDataContainer;
import org.processmining.models.queueminer.queuepredictor.QueuePredictor;
import org.processmining.models.queueminer.utils.ArrivingEvent;

@Plugin(name = "Make exit time predictions based on a queueing model", returnLabels = { "Prediction data" }, returnTypes = { PredictionDataContainer.class }, parameterLabels = {
		"XES Log", "QueueClustering", "Parameters" }, userAccessible = true)
public class QueueExitTimePredictorPlugin {
	
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "L.P.J. de Smet", email = "l.p.j.d.smet@student.tue.nl")
	@PluginVariant(variantLabel = "Execute queue-based predictions", requiredParameterLabels = { 0,1})
	public  PredictionDataContainer findExitTimePredictions(UIPluginContext context,XLog testingData, QueueClustering classifier){
		//put test data in convenient format
		ArrayList<ArrivingEvent> testValues = new ArrayList<>();
		for(XTrace tr:testingData){
			for(XEvent ev:tr){
				testValues.add(ArrivingEvent.fromLogEvent(ev));
			}
		}
		//Then for all arriving events predict the exit time
		PredictionDataContainer pdc = new PredictionDataContainer();
		for(ArrivingEvent testV:testValues){
			QueuePredictor.predictQueueExitTime(classifier, testV, 0);
			pdc.addPredictionData(testV);
		}
		//then return the result
		return pdc;
	}
}
