package org.processmining.plugins.queueminer.queueratefinder;

import java.util.ArrayList;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XLog;
import org.processmining.confs.queueminer.queueratefinder.QueueRateFinderConfig;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.dialogs.queueminer.queueratefinder.QueueRateFinderDialog;
import org.processmining.framework.connections.Connection;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.queueminer.matrixfinder.ResActMatrix;
import org.processmining.models.queueminer.queueclusterer.QueueClustering;
import org.processmining.models.queueminer.queueratefinder.QueueRateFinder;
import org.processmining.models.queueminer.utils.Queue;

@Plugin(name = "Queue rate finder", returnLabels = { "Queue with rates" }, returnTypes = { QueueClustering.class }, parameterLabels = {
		"XES Log (with context)", "QueueClustering", "Parameters" }, userAccessible = false)
public class QueueRateFinderPlugin {
	
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "L.P.J. de Smet", email = "l.p.j.d.smet@student.tue.nl")
	@PluginVariant(variantLabel = "Queue rate finder", requiredParameterLabels = { 0,1})
	public QueueClustering findCalendar(UIPluginContext context, XLog log, QueueClustering qc) {
		return regularFindCalendar(context,log,qc);
	}
	
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "L.P.J. de Smet", email = "l.p.j.d.smet@student.tue.nl")
	@PluginVariant(variantLabel = "Queue rate finder, All-in-one", requiredParameterLabels = { 0})
	public QueueClustering findCalendarAllInOne(UIPluginContext context, XLog log) {
		ResActMatrix ram;
		QueueClustering qc = new QueueClustering(new ArrayList<Queue>());
		try {
			ram = context.tryToFindOrConstructFirstNamedObject(ResActMatrix.class, "Resource activity matrix finder", Connection.class, "", log);
			Object[] input = new Object[2];
			input[0] = log;
			input[1] = ram;
			qc = context.tryToFindOrConstructFirstNamedObject(QueueClustering.class, "Queue clusterer", Connection.class, "", input);
		} catch (ConnectionCannotBeObtained e) {
			e.printStackTrace();
		}
		return regularFindCalendar(context,log,qc);
	}
	
	public QueueClustering regularFindCalendar(UIPluginContext context, XLog log, QueueClustering qc){
		QueueRateFinderConfig qrfc = new QueueRateFinderConfig(1);
		QueueRateFinderDialog qrfd = new QueueRateFinderDialog(qrfc);
		//ask user for wanted k
		InteractionResult result = context.showWizard("Queue rate finder config", true, true, qrfd);
		if (result == InteractionResult.FINISHED) {
			QueueClustering updatedQueue = QueueRateFinder.findQueueClustering(log, qc,qrfc.k,context.getProgress());
			return updatedQueue;
		}
		//something went terribly wrong
		return null;
	}
	
	
}
