package org.processmining.plugins.queueminer.sojournpredictor;

import java.net.URI;
import java.util.ArrayList;
import java.util.Collection;

import org.deckfour.xes.extension.XExtension;
import org.deckfour.xes.extension.XExtensionManager;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.Connection;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.queueminer.queueclusterer.QueueClustering;
import org.processmining.models.queueminer.sojournpredictor.QueueSojournPredictor;
import org.processmining.models.queueminer.sojournpredictor.SojPredictionDataContainer;
import org.processmining.models.queueminer.utils.ArrivingEvent;
import org.processmining.models.queueminer.utils.ArrivingTrace;
import org.processmining.models.queueminer.utils.Queue;

@Plugin(name = "Make sojourn time predictions based on a queueing model", returnLabels = { "Prediction data" }, returnTypes = { SojPredictionDataContainer.class }, parameterLabels = {
		"XLog Training data","XLog Testing data", "QueueClustering", "Parameters" }, userAccessible = true)
public class QueueSojournTimePredictorPlugin {
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "L.P.J. de Smet", email = "l.p.j.d.smet@student.tue.nl")
	@PluginVariant(variantLabel = "Execute queue-based sojourn predictions", requiredParameterLabels = { 0,1 })
	public SojPredictionDataContainer findErrorOfPredictionAIO(UIPluginContext context, XLog trainingData, XLog testingData) throws Exception {
		QueueClustering qc = new QueueClustering(new ArrayList<Queue>());
		try {
			qc = context.tryToFindOrConstructFirstNamedObject(QueueClustering.class, "Queue rate finder", Connection.class, "", trainingData);
		} catch (ConnectionCannotBeObtained e) {
			e.printStackTrace();
		}
		if(qc.getQueues().isEmpty()){
			return null;
		}
		return findExitTimePredictions(context,trainingData,testingData,qc);
	}
	
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "L.P.J. de Smet", email = "l.p.j.d.smet@student.tue.nl")
	@PluginVariant(variantLabel = "Execute queue-based predictions", requiredParameterLabels = { 0,1,2})
	public  SojPredictionDataContainer findExitTimePredictions(UIPluginContext context,XLog trainingData, XLog testingData, QueueClustering classifier){
		//put test data in convenient format
		ArrayList<ArrivingTrace> testValues = new ArrayList<>();
		for(XTrace tr:testingData){
			testValues.add(ArrivingTrace.fromLogTrace(tr));
		}
		//Then for all arriving events predict the exit time
		SojPredictionDataContainer pdc = new SojPredictionDataContainer();
		QueueSojournPredictor qsp = new QueueSojournPredictor(trainingData,classifier.getK(),classifier);
		for(ArrivingTrace testV:testValues){
			//find arriving event from testV
			ArrivingEvent ae = new ArrivingEvent(testV.eventType, testV.arrivalTime, testV.context, testV.getEventID());
			//check whether context information is used
			//get first event of training data
			XEvent firstEv = trainingData.get(0).get(0);
			//check on context info
			XExtensionManager xem = XExtensionManager.instance();
			URI contextUri = URI.create("http://www.zerodivision.nl/XES/context.xesext");
			XExtension xcoe = xem.getByUri(contextUri);
			Collection<XAttribute> attributes = xcoe.getEventAttributes();
			XAttribute contattr = attributes.iterator().next();
			boolean useContext = false;
			String cont = "Regular";
			try{
				cont = firstEv.getAttributes().get(contattr.getKey()).toString();
				useContext = true;
			}
			catch(Exception e){
				useContext = false;
			}
			
			testV.predictTime(qsp.predictSojournTime(ae,useContext));
			pdc.addPredictionData(testV);
		}
		//then return the result
		return pdc;
	}
	
}
